"""
This plugin extends Gaphor with XMI alignment actions.
"""

from zope import interface, component
from gaphor.core import inject, transactional, action, build_action_group
from gaphor.interfaces import IService, IActionProvider
from gaphor.ui.interfaces import IDiagramSelectionChange


class Alignment(object):

    interface.implements(IService, IActionProvider)

    component_registry = inject('component_registry')

    menu_xml = """
      <ui>
        <menubar action="mainwindow">
          <menu action="diagram">
            <placeholder name="ternary">
		<separator />
		<menuitem action="align-left" />
		<menuitem action="align-center" />
		<menuitem action="align-right" />
		<separator />
		<menuitem action="align-top" />
		<menuitem action="align-middle" />
		<menuitem action="align-bottom" />
            </placeholder>
          </menu>
        </menubar>
      </ui>"""

    def __init__(self):
        self.action_group = build_action_group(self)
        self._last_update = None

    def init(self, app):
        self.component_registry.register_handler(self.update)
    
    def shutdown(self):
        self.component_registry.unregister_handler(self.update)

    @component.adapter(IDiagramSelectionChange)
    def update(self, event=None):
        self._last_update = event
        sensitive = event and len(event.selected_items) > 1
	self.action_group.get_action('align-left').set_sensitive(sensitive)
	self.action_group.get_action('align-center').set_sensitive(sensitive)
	self.action_group.get_action('align-right').set_sensitive(sensitive)
	self.action_group.get_action('align-top').set_sensitive(sensitive)
	self.action_group.get_action('align-middle').set_sensitive(sensitive)
	self.action_group.get_action('align-bottom').set_sensitive(sensitive)

    def get_items(self):
        return (self._last_update and self._last_update.selected_items) or []
            
    def getXCoordsLeft(self, items):
        return [item.matrix[4] for item in items]
	
    def getXCoordsRight(self, items):
        return [item.matrix[4] + item.width for item in items]

    def getYCoordsTop(self, items):
        return [item.matrix[5] for item in items]
	
    def getYCoordsBottom(self, items):
        return [item.matrix[5] + item.height for item in items]

    @action(name='align-left', label='Left',
            tooltip="Vertically align diagram elements on the left")
    @transactional
    def align_left(self):
        items = self.get_items()
        target_x=min(self.getXCoordsLeft(items))
        for item in items:
            x = target_x - item.matrix[4]
            item.matrix.translate(x,0)
            item.request_update()
	
    @action(name='align-center', label='Center',
            tooltip="Vertically align diagram elements on their centers")
    @transactional
    def align_center(self):
        items = self.get_items()
        min_x=min(self.getXCoordsLeft(items))
	max_x=max(self.getXCoordsRight(items))
	center_x = max_x - min_x
        for item in items:
            x = center_x - (item.width / 2) - item.matrix[4]
            item.matrix.translate(x,0)
            item.request_update()
	
    @action(name='align-right', label='Right',
            tooltip="Vertically align diagram elements on the right")
    @transactional
    def align_right(self):
        items = self.get_items()
        target_x=max(self.getXCoordsRight(items))
        for item in items:
            x = target_x - item.width - item.matrix[4]
            item.matrix.translate(x,0)
            item.request_update()

    @action(name='align-top', label='Top',
            tooltip="Horizontally align diagram elements on their tops")
    @transactional
    def align_top(self):
        items = self.get_items()
        target_y = min(self.getYCoordsTop(items))
        for item in items:
            y = target_y - item.matrix[5]
            item.matrix.translate(0,y)
            item.request_update()
	    
    @action(name='align-middle', label='Middle',
            tooltip="Horizontally align diagram elements on their middles")
    @transactional
    def align_middle(self):
        items = self.get_items()
        min_y = min(self.getYCoordsTop(items))
	max_y = max(self.getYCoordsBottom(items))
	middle_y = max_y - min_y
        for item in items:
	    y = middle_y - (item.height / 2) - item.matrix[5]
            item.matrix.translate(0,y)
            item.request_update()
	    
    @action(name='align-bottom', label='Bottom',
            tooltip="Horizontally align diagram elements on their bottoms")
    @transactional
    def align_bottom(self):
        items = self.get_items()
        target_y = min(self.getYCoordsBottom(items))
        for item in items:
            y = target_y - item.height - item.matrix[5]
            item.matrix.translate(0,y)
            item.request_update()
	


# vim:sw=4:et:ai



# vim:sw=4:et

"""The code reverse engineer.
"""

from zope import component
from gaphor import UML
from gaphor.diagram import items
from gaphor.core import inject
from gaphor.diagram.interfaces import IConnect
from gaphas.aspect import ConnectionSink, Connector

from pynsource import PySourceAsText

BASE_CLASSES = ('object', 'type', 'dict', 'list', 'tuple', 'int', 'float')

class Engineer(object):
    """
    The Engineer class will create a Gaphor model based on a list of Python
    files.
    """

    element_factory = inject('element_factory')
    diagram_layout = inject('diagram_layout')

    def process(self, files=None):

        # these are tuples between class names.
        #self.associations_generalisation = []
        #self.associations_composition = []

        p = PySourceAsText()
        self.parser = p

        if files:
            #u = PythonToJava(None, treatmoduleasclass=0, verbose=0)
            for f in files:
                # Build a shape with all attrs and methods, and prepare association dict
                p.Parse(f)

        print p
        
        try:
            self._root_package = self.element_factory.lselect(lambda e: isinstance(e, UML.Package) and not e.namespace)[0]
        except IndexError:
            pass # running as test?

        for m in p.modulemethods:
            print 'ModuleMethod:', m

        # Step 0: create a diagram to put the newly created elements on
        self.diagram = self.element_factory.create(UML.Diagram)
        self.diagram.name = 'New classes'
        self.diagram.package = self._root_package

        # Step 1: create the classes
        for name, clazz in p.classlist.items():
            print type(clazz), dir(clazz)
            self._create_class(clazz, name)
            
        # Create generalization relationships:
        for name, clazz in p.classlist.items():
            self._create_generalization(clazz)
        
        # Create attributes (and associations) on the classes
        for name, clazz in p.classlist.items():
            self._create_attributes(clazz)

        # Create operations
        for name, clazz in p.classlist.items():
            self._create_methods(clazz)

        self.diagram_layout.layout_diagram(self.diagram)

    def _create_class(self, clazz, name):
        c = self.element_factory.create(UML.Class)
        c.name = name
        c.package = self.diagram.namespace
        ci = self.diagram.create(items.ClassItem)
        ci.subject = c
        clazz.gaphor_class = c
        clazz.gaphor_class_item = ci

    def _create_generalization(self, clazz):
        if not clazz.ismodulenotrealclass:
            for superclassname in clazz.classesinheritsfrom:
                if superclassname in BASE_CLASSES:
                    continue
                try:
                    superclass = self.parser.classlist[superclassname].gaphor_class
                    superclass_item = self.parser.classlist[superclassname].gaphor_class_item
                except KeyError, e:
                    print 'No class found named', superclassname
                    others = self.element_factory.lselect(lambda e: isinstance(e, UML.Class) and e.name == superclassname)
                    if others:
                        superclass = others[0]
                        print 'Found class in factory: %s' % superclass.name
                        superclass_item = self.diagram.create(items.ClassItem)
                        superclass_item.subject = superclass
                    else:
                        continue
                # Finally, create the generalization relationship
                print 'Creating Generalization for %s' % clazz, superclass
                #gen = self.element_factory.create(UML.Generalization)
                #gen.general = superclass
                #gen.specific = clazz.gaphor_class
                geni = self.diagram.create(items.GeneralizationItem)
                #geni.subject = gen
                
                self.connect(geni, geni.tail, clazz.gaphor_class_item)
                self.connect(geni, geni.head, superclass_item)
                
                #adapter = component.queryMultiAdapter((superclass_item, geni), IConnect)
                #assert adapter
                #handle = geni.handles()[0]
                #adapter.connect(handle)
                #clazz.gaphor_class_item.connect_handle(geni.handles[-1])
                #adapter = component.queryMultiAdapter((clazz.gaphor_class_item, geni), IConnect)
                #assert adapter
                #handle = geni.handles()[-1]
                #adapter.connect(handle)
                
    def connect(self, line, handle, item, port=None):
        """
        Connect line's handle to an item.

        If port is not provided, then first port is used.
        """
        canvas = line.canvas

        if port is None and len(item.ports()) > 0:
            port = item.ports()[0]

        sink = ConnectionSink(item, port)
        connector = Connector(line, handle)

        connector.connect(sink)

    def _create_attributes(self, clazz):
        for attrobj in clazz.attrs:
            # TODO: Check object type and figure out if it should be an
            # attribute or an association.
            self._create_attribute(clazz, attrobj)

    def _create_methods(self, clazz):
        for adef in clazz.defs:
            op = self.element_factory.create(UML.Operation)
            op.name = adef
            clazz.gaphor_class.ownedOperation = op

    def _find_class_by_name(self, classname):
        try:
            superclass = self.parser.classlist[classname].gaphor_class
            superclass_item = self.parser.classlist[classname].gaphor_class_item
        except KeyError, e:
            print 'No class found named', classname
            others = self.element_factory.lselect(lambda e: isinstance(e, UML.Class) and e.name == classname)
            if others:
                superclass = others[0]
                print 'Found class in factory: %s' % superclass.name
                superclass_item = self.diagram.create(items.ClassItem)
                superclass_item.subject = superclass
            else:
                return None, None
        return superclass, superclass_item

    def _visibility(self, attrname):
        if attrname.startswith('__'):
            return 'private'
        elif attrname.startswith('_'):
            return 'protected'
        return 'public'

    def _create_attribute(self, clazz, attr):
        static = False
        many = False
        if 'static' in attr.attrtype:
            static = True
        if 'many' in attr.attrtype:
            many = True
        compositescreated = self.parser.GetCompositeClassesForAttr(attr.attrname, clazz)
        tail_type = None
        if compositescreated:
            tail_type, tail_type_item = self._find_class_by_name(compositescreated[0])

        if tail_type:
            # Create an association:
            #print "%s %s <@>----> %s" % (attr.attrname, static, str(compositescreated))
            # The property on the tail of the association (tail_end) is owned
            # by the class connected on the head_end (head_type)
            head_type = clazz.gaphor_class
            head_type_item = clazz.gaphor_class_item

            #relation = self.element_factory.create(UML.Association)
            #head_end = self.element_factory.create(UML.Property)
            #head_end.lowerValue = self.element_factory.create(UML.LiteralSpecification)
            #tail_end = self.element_factory.create(UML.Property)
            #tail_end.name = attr.attrname
            #tail_end.visibility = self._visibility(attr.attrname)
            #tail_end.aggregation = 'composite'
            #tail_end.lowerValue = self.element_factory.create(UML.LiteralSpecification)
            #relation.package = self.diagram.namespace
            #relation.memberEnd = head_end
            #relation.memberEnd = tail_end
            #head_end.type = head_type
            #tail_end.type = tail_type
            #head_type.ownedAttribute = tail_end
            #tail_type.ownedAttribute = head_end

            
            # Now the subject
            #association.subject = relation
            #association.head_end.subject = head_end
            #association.tail_end.subject = tail_end

            # Create the diagram item:
            association = self.diagram.create(items.AssociationItem)

            adapter = component.queryMultiAdapter((head_type_item, association), IConnect)
            assert adapter
            handle = association.handles()[0]
            adapter.connect(handle)

            adapter = component.queryMultiAdapter((tail_type_item, association), IConnect)
            assert adapter
            handle = association.handles()[-1]
            adapter.connect(handle)

            # Apply attribute information to the association (ends)
            association.head_end.navigability = False
            tail_prop = association.tail_end.subject
            tail_prop.name = attr.attrname
            tail_prop.visibility = self._visibility(attr.attrname)
            tail_prop.aggregation = 'composite'
        else:
            # Create a simple attribute:
            #print "%s %s" % (attr.attrname, static)
            prop = self.element_factory.create(UML.Property)
            prop.name = attr.attrname
            prop.visibility = self._visibility(attr.attrname)
            prop.isStatic = static
            clazz.gaphor_class.ownedAttribute = prop
        #print many
        import pprint
        pprint.pprint(attr)
        #print dir(attr)




"""
Interfaces related to the user interface.
"""

from zope import interface


class IDiagramTabChange(interface.Interface):
    """
    The selected diagram changes.
    """
    item = interface.Attribute('The newly selected DockItem')

    diagram_tab = interface.Attribute('The newly selected diagram tab')

class IDiagramSelectionChange(interface.Interface):
    """
    The selection of a diagram changed.
    """
    diagram_view = interface.Attribute('The diagram View that emits the event')

    focused_item = interface.Attribute('The diagram item that received focus')

    selected_items = interface.Attribute('All selected items in the diagram')


class IUIComponent(interface.Interface):
    """
    A user interface component.
    """
    
    ui_name = interface.Attribute('The UIComponent name, provided by the loader')

    title = interface.Attribute('Title of the component')

    size = interface.Attribute('Size used for floating the component')

    placement = interface.Attribute('placement. E.g. ("left", "diagrams")')

    def open(self):
        """
        Create and display the UI components (windows).
        """

    def close(self):
        """
        Close the UI component. The component can decide to hide or destroy the UI
        components.
        """


class IPropertyPage(interface.Interface):
    """
    A property page which can display itself in a notebook
    """
    
    order = interface.Attribute('Order number, used for ordered display')

    def construct(self):
        """
        Create the page (gtk.Widget) that belongs to the Property page.

        Returns the page's toplevel widget (gtk.Widget).
        """

    def destroy(self):
        """
        Destroy the page and clean up signal handlers and stuff.
        """


# vim:sw=4:et:ai



"""
Test include item connections.
"""

from gaphor.tests import TestCase
from gaphor import UML
from gaphor.diagram import items

class IncludeItemTestCase(TestCase):
    def test_use_case_glue(self):
        """Test "include" glueing to use cases
        """
        uc1 = self.create(items.UseCaseItem, UML.UseCase)
        include = self.create(items.IncludeItem)

        glued = self.allow(include, include.head, uc1)
        self.assertTrue(glued)


    def test_use_case_connect(self):
        """Test connecting "include" to use cases
        """
        uc1 = self.create(items.UseCaseItem, UML.UseCase)
        uc2 = self.create(items.UseCaseItem, UML.UseCase)
        include = self.create(items.IncludeItem)

        self.connect(include, include.head, uc1)
        self.assertTrue(self.get_connected(include.head), uc1)

        self.connect(include, include.tail, uc2)
        self.assertTrue(self.get_connected(include.tail), uc2)


    def test_use_case_connect(self):
        """Test reconnecting use cases with "include"
        """
        uc1 = self.create(items.UseCaseItem, UML.UseCase)
        uc2 = self.create(items.UseCaseItem, UML.UseCase)
        uc3 = self.create(items.UseCaseItem, UML.UseCase)
        include = self.create(items.IncludeItem)

        # connect: uc1 -> uc2
        self.connect(include, include.head, uc1)
        self.connect(include, include.tail, uc2)
        e = include.subject

        # reconnect: uc1 -> uc2
        self.connect(include, include.tail, uc3)

        self.assertSame(e, include.subject)
        self.assertSame(include.subject.addition, uc1.subject)
        self.assertSame(include.subject.includingCase, uc3.subject)


    def test_use_case_disconnect(self):
        """Test disconnecting "include" from use cases
        """
        uc1 = self.create(items.UseCaseItem, UML.UseCase)
        uc2 = self.create(items.UseCaseItem, UML.UseCase)
        include = self.create(items.IncludeItem)

        self.connect(include, include.head, uc1)
        self.connect(include, include.tail, uc2)

        self.disconnect(include, include.head)
        self.assertTrue(self.get_connected(include.head) is None)
        self.assertTrue(include.subject is None)

        self.disconnect(include, include.tail)
        self.assertTrue(self.get_connected(include.tail) is None)



# vim:sw=4:et:ai



"""
The file service is responsible for loading and saving the user data.
"""

from logging import getLogger

import gtk
from zope import interface, component

from gaphor.interfaces import IService, IActionProvider, IServiceEvent
from gaphor.core import _, inject, action, build_action_group
from gaphor.storage import storage, verify
from gaphor import UML
from gaphor.misc.gidlethread import GIdleThread, Queue, QueueEmpty
from gaphor.misc.errorhandler import error_handler
from gaphor.misc.xmlwriter import XMLWriter
from gaphor.ui.statuswindow import StatusWindow
from gaphor.ui.questiondialog import QuestionDialog
from gaphor.ui.filedialog import FileDialog

DEFAULT_EXT = '.gaphor'
MAX_RECENT = 10

class FileManagerStateChanged(object):
    """
    Event class used to send state changes on the ndo Manager.
    """

    interface.implements(IServiceEvent)

    def __init__(self, service):
        self.service = service


class FileManager(object):
    """
    The file service, responsible for loading and saving Gaphor models.
    """

    interface.implements(IService, IActionProvider)

    component_registry = inject('component_registry')
    element_factory = inject('element_factory')
    main_window = inject('main_window')
    properties = inject('properties')
    logger = getLogger('FileManager')

    menu_xml = """
      <ui>
        <menubar name="mainwindow">
          <menu action="file">
            <placeholder name="primary">
              <menuitem action="file-new" />
              <menuitem action="file-new-template" />
              <menuitem action="file-open" />
              <menu name="recent" action="file-recent-files">
                <menuitem action="file-recent-0" />
                <menuitem action="file-recent-1" />
                <menuitem action="file-recent-2" />
                <menuitem action="file-recent-3" />
                <menuitem action="file-recent-4" />
                <menuitem action="file-recent-5" />
                <menuitem action="file-recent-6" />
                <menuitem action="file-recent-7" />
                <menuitem action="file-recent-8" />
              </menu>
              <separator />
              <menuitem action="file-save" />
              <menuitem action="file-save-as" />
              <separator />
            </placeholder>
          </menu>
        </menubar>
        <toolbar action="mainwindow-toolbar">
          <placeholder name="left">
            <toolitem action="file-open" />
            <separator />
            <toolitem action="file-save" />
            <toolitem action="file-save-as" />
            <separator />
          </placeholder>
        </toolbar>
      </ui>
    """

    def __init__(self):
        """File manager constructor.  There is no current filename yet."""

        self._filename = None

    def init(self, app):
        """File manager service initialization.  The app parameter
        is the main application object.  This method builds the
        action group in the file menu.  The list of recent
        Gaphor files is then updated in the file menu."""
    
        self.action_group = build_action_group(self)

        for name, label in (('file-recent-files', '_Recent files'),):
            action = gtk.Action(name, label, None, None)
            action.set_property('hide-if-empty', False)
            self.action_group.add_action(action)

        for i in xrange(0, (MAX_RECENT-1)):
            action = gtk.Action('file-recent-%d' % i, None, None, None)
            action.set_property('visible', False)
            self.action_group.add_action(action)
            action.connect('activate', self.load_recent, i)
            
        self.update_recent_files()

    def shutdown(self):
        """Called when shutting down the file manager service."""

        self.logger.info('Shutting down')
        
    def get_filename(self):
        """Return the current file name.  This method is used by the filename
        property."""
        return self._filename

    def set_filename(self, filename):
        """Sets the current file name.  This method is used by the filename
        property.  Setting the current filename will update the recent file
        list."""

        self.logger.info('Setting current file')
        self.logger.debug('Filename is %s' % filename)

        if filename != self._filename:
            self._filename = filename
            self.update_recent_files(filename)

    filename = property(get_filename, set_filename)
    
    def get_recent_files(self):
        """Returns the recent file list from the properties service.  This
        method is used by the recent_files property."""
        
        try:
            return self.properties.get('recent-files', [])
        except component.interfaces.ComponentLookupError:
            return []
        
    def set_recent_files(self, recent_files):
        """Updates the properties service with the supplied list of recent 
        files.  This method is used by the recent_files property."""
        
        self.logger.info('Storing recent files')
        self.logger.debug('Recent files are %s' % recent_files)
        
        try:
            self.properties.set('recent-files', recent_files)
        except component.interfaces.ComponentLookupError:
            return
        
    recent_files = property(get_recent_files, set_recent_files)

    def update_recent_files(self, new_filename=None):
        """Updates the list of recent files.  If the new_filename
        parameter is supplied, it is added to the list of recent files.
        
        The default recent file placeholder actions are hidden.  The real
        actions are then built using the recent file list."""
        
        self.logger.info('Updating recent files')
        self.logger.debug('New file is %s' % new_filename)

        recent_files = self.recent_files
        
        if new_filename and new_filename not in recent_files:
            recent_files.insert(0, new_filename)
            recent_files = recent_files[0:(MAX_RECENT-1)]
            self.recent_files = recent_files

        for i in xrange(0, (MAX_RECENT-1)):
            action = self.action_group.get_action('file-recent-%d' % i)
            action.set_property('visible', False)

        for i, filename in enumerate(recent_files):
            id = 'file-recent%d' % i
            action = self.action_group.get_action('file-recent-%d' % i)
            action.props.label = '_%d. %s' % (i+1, filename.replace('_', '__'))
            action.props.tooltip = 'Load %s.' % filename
            action.props.visible = True

    def load_recent(self, action, index):
        """Load the recent file at the specified index.  This will trigger
        a FileManagerStateChanged event.  The recent files are stored in
        the recent_files property."""
        
        self.logger.info('Loading recent file')
        self.logger.debug('Action is %s' % action)
        self.logger.debug('Index is %s' % index)

        filename = self.recent_files[index]

        self.load(filename)
        self.component_registry.handle(FileManagerStateChanged(self))
        

    def load(self, filename):
        """Load the Gaphor model from the supplied file name.  A status window
        displays the loading progress.  The load generator updates the progress
        queue.  The loader is passed to a GIdleThread which executes the load
        generator.  If loading is successful, the filename is set."""

        self.logger.info('Loading file')
        self.logger.debug('Path is %s' % filename)

        queue = Queue()
        
        try:
            main_window = self.main_window
            status_window = StatusWindow(_('Loading...'),\
                                         _('Loading model from %s') % filename,\
                                         parent=main_window.window,\
                                         queue=queue)            
        except component.interfaces.ComponentLookupError:
            status_window = None

        try:
            loader = storage.load_generator(filename.encode('utf-8'), self.element_factory)
            worker = GIdleThread(loader, queue)

            worker.start()
            worker.wait()
            
            if worker.error:
                worker.reraise()

            self.filename = filename
        except:
            error_handler(message=_('Error while loading model from file %s') % filename)
            raise
        finally:
            if status_window is not None:
                status_window.destroy()

        
    def verify_orphans(self):
        """Verify that no orphaned elements are saved.  This method checks
        of there are any orphan references in the element factory.  If orphans
        are found, a dialog is displayed asking the user if it is OK to
        unlink them."""
        
        orphans = verify.orphan_references(self.element_factory)
        
        if orphans:
            main_window = self.main_window

            dialog = QuestionDialog(_("The model contains some references"\
                                      " to items that are not maintained."\
                                      " Do you want to clean this before"\
                                      " saving the model?"),\
                                    parent=main_window.window)
          
            answer = dialog.answer
            dialog.destroy()
            
            if not answer:
                for orphan in orphans:
                    orphan.unlink()
                    
    def verify_filename(self, filename):
        """Verify that the supplied filename is using the proper default
        extension.  If not, the extension is added to the filename
        and returned."""
        
        self.logger.debug('Verifying file name')
        self.logger.debug('File name is %s' % filename)
        
        if not filename.endswith(DEFAULT_EXT):
            filename = filename + DEFAULT_EXT
            
        return filename

    def save(self, filename):
        """Save the current UML model to the specified file name.  Before
        writing the model file, this will verify that there are no orphan
        references.  It will also verify that the filename has the correct
        extension.  A status window is displayed while the GIdleThread
        is executed.  This thread actually saves the model."""
        
        self.logger.info('Saving file')
        self.logger.debug('File name is %s' % filename)

        if not filename or not len(filename):
            return

        self.verify_orphans()
        filename = self.verify_filename(filename)

        main_window = self.main_window
        queue = Queue()
        status_window = StatusWindow(_('Saving...'),\
                                     _('Saving model to %s') % filename,\
                                     parent=main_window.window,\
                                     queue=queue)
        try:
            with open(filename.encode('utf-8'), 'w') as out:
                saver = storage.save_generator(XMLWriter(out), self.element_factory)
                worker = GIdleThread(saver, queue)
                worker.start()
                worker.wait()
            
            if worker.error:
                worker.reraise()
                
            
            self.filename = filename
        except:
            error_handler(message=_('Error while saving model to file %s') % filename)
            raise
        finally:
            status_window.destroy()

    def _open_dialog(self, title):
        """Open a file chooser dialog to select a model
        file to open."""

        filesel = gtk.FileChooserDialog(title=title,
                                        action=gtk.FILE_CHOOSER_ACTION_OPEN,
                                        buttons=(gtk.STOCK_CANCEL,
                                                 gtk.RESPONSE_CANCEL,
                                                 gtk.STOCK_OPEN,
                                                 gtk.RESPONSE_OK))
        filesel.set_transient_for(self.main_window.window)

        filter = gtk.FileFilter()
        filter.set_name("Gaphor models")
        filter.add_pattern("*.gaphor")
        filesel.add_filter(filter)

        filter = gtk.FileFilter()
        filter.set_name("All files")
        filter.add_pattern("*")
        filesel.add_filter(filter)

        if self.filename:
            filesel.set_current_name(self.filename)

        response = filesel.run()
        filename = filesel.get_filename()
        filesel.destroy()
        if not filename or response != gtk.RESPONSE_OK:
            return
        return filename

    @action(name='file-new', stock_id='gtk-new')
    def action_new(self):
        """The new model menu action.  This action will create a new
        UML model.  This will trigger a FileManagerStateChange event."""

        element_factory = self.element_factory
        main_window = self.main_window

        if element_factory.size():
            dialog = QuestionDialog(_("Opening a new model will flush the"\
                                      " currently loaded model.\nAny changes"\
                                      " made will not be saved. Do you want to"\
                                      " continue?"),\
                                    parent=main_window.window)
           
            answer = dialog.answer
            dialog.destroy()
            
            if not answer:
                return

        element_factory.flush()
        model = element_factory.create(UML.Package)
        model.name = _('New model')
        diagram = element_factory.create(UML.Diagram)
        diagram.package = model
        diagram.name= _('main')
        self.filename = None
        element_factory.notify_model()

        #main_window.select_element(diagram)
        #main_window.show_diagram(diagram)

        self.component_registry.handle(FileManagerStateChanged(self))

    @action(name='file-new-template', label=_('New from template'))
    def action_new_from_template(self):
        """This menu action opens the new model from template dialog."""

        filters = [{'name':_('Gaphor Models'), 'pattern':'*.gaphor'},\
                   {'name':_('All Files'), 'pattern':'*'}]

        file_dialog = FileDialog(_('New Gaphor Model From Template'),\
                                 filters = filters)
        
        filename = file_dialog.selection
        
        file_dialog.destroy()
        
        log.debug(filename)

        if filename:
            self.load(filename)
            self.filename = None
            self.component_registry.handle(FileManagerStateChanged(self))


    @action(name='file-open', stock_id='gtk-open')
    def action_open(self):
        """This menu action opens the standard model open dialog."""

        filters = [{'name':_('Gaphor Models'), 'pattern':'*.gaphor'},\
                   {'name':_('All Files'), 'pattern':'*'}]

        file_dialog = FileDialog(_('Open Gaphor Model'),\
                                 filters = filters)
        
        filename = file_dialog.selection
        
        file_dialog.destroy()

        log.debug(filename)

        if filename:
            self.load(filename)
            self.component_registry.handle(FileManagerStateChanged(self))


    @action(name='file-save', stock_id='gtk-save')
    def action_save(self):
        """
        Save the file. Depending on if there is a file name, either perform
        the save directly or present the user with a save dialog box.

        Returns True if the saving actually succeeded.
        """

        filename = self.filename

        if filename:
            self.save(filename)
            self.component_registry.handle(FileManagerStateChanged(self))
            return True
        else:
            return self.action_save_as()


    @action(name='file-save-as', stock_id='gtk-save-as')
    def action_save_as(self):
        """
        Save the model in the element_factory by allowing the
        user to select a file name.

        Returns True if the saving actually happened.
        """

        file_dialog = FileDialog(_('Save Gaphor Model As'),\
                                 action='save',\
                                 filename=self.filename)
        
        filename = file_dialog.selection
        
        file_dialog.destroy()
        
        if filename:
            self.save(filename)
            self.component_registry.handle(FileManagerStateChanged(self))
            return True

        return False

# vim:sw=4:et:ai



"""This module has a generic FileDialog class that is used to open
or save files."""

import gtk

class FileDialog(object):
    """This is a file dialog that is used to open or save a file."""
    
    def __init__(self, title, filename=None, action='open', parent=None, multiple=False, filters=[]):
        """Initialize the file dialog.  The title parameter is the title
        displayed in the dialog.  The filename parameter will set the current
        file name in the dialog.  The action is either open or save and changes
        the buttons isplayed.  If the parent window parameter is supplied,
        the file dialog is set to be transient for that window.  The multiple
        parameter should be set to true if sultiple files can be opened at once.
        This means that a list of filenames instead of a single filename string
        will be returned by the selection property.  The filters is a list
        of dictionaries that have a name and pattern key.  This restricts what
        is visible in the dialog."""
        
        self.multiple = multiple
        
        if action == 'open':
            action = gtk.FILE_CHOOSER_ACTION_OPEN
            response_button = gtk.STOCK_OPEN
        else:
            action = gtk.FILE_CHOOSER_ACTION_SAVE
            response_button = gtk.STOCK_SAVE
            
        buttons = (gtk.STOCK_CANCEL,\
                   gtk.RESPONSE_CANCEL,\
                   response_button,\
                   gtk.RESPONSE_OK)
        
        self.dialog = gtk.FileChooserDialog(title=title,
                                            action=action,
                                            buttons=buttons)
                                            
        if parent:
            self.dialog.set_transient_for(parent)
            
        if filename:
            self.dialog.set_current_name(filename)
            
        for filter in filters:
            _filter = gtk.FileFilter()
            _filter.set_name(filter['name'])
            _filter.add_pattern(filter['pattern'])
            self.dialog.add_filter(_filter)
        
    def get_selection(self):
        """Return the selected file or files from the dialog.  This is used
        by the selection property."""
        
        response = self.dialog.run()
        selection = None
        
        if response == gtk.RESPONSE_OK:
            
            if self.multiple:
                selection = self.dialog.get_filenames()
            else:
                selection = self.dialog.get_filename()
            
        return selection
        
    def destroy(self):
        """Destroy the GTK dialog."""
        
        self.dialog.destroy()
                
    selection = property(get_selection)



"""
"""

import gtk
from zope import interface, component

from logging import getLogger
from gaphor.core import inject
from gaphor.interfaces import IService, IActionProvider
from gaphor.event import ServiceInitializedEvent, ActionExecuted

class ActionManager(object):
    """
    This service is responsible for maintaining actions.
    """

    interface.implements(IService)
    logger = getLogger('ActionManager')

    component_registry = inject('component_registry')

    ui_manager = inject('ui_manager')

    def __init__(self):
        pass


    def init(self, app):
        self.logger.info('Loading action provider services')
        
        for name, service in self.component_registry.get_utilities(IActionProvider):
            self.logger.debug('Service is %s' % service)
            self.register_action_provider(service)

        self.component_registry.register_handler(self._service_initialized_handler)

    def shutdown(self):
        
        self.logger.info('Shutting down')
        
        self.component_registry.unregister_handler(self._service_initialized_handler)

    def execute(self, action_id, active=None):
        
        self.logger.debug('Executing action, action_id is %s' % action_id)
        
        a = self.get_action(action_id)
        if a:
            a.activate()
            self.component_registry.handle(ActionExecuted(action_id, a))
        else:
            self.logger.warning('Unknown action %s' % action_id)

    def update_actions(self):
        
        self.ui_manager.ensure_update()

    def get_action(self, action_id):
        
        for g in self.ui_manager.get_action_groups():
            a = g.get_action(action_id)
            if a: return a

    def register_action_provider(self, action_provider):
        
        self.logger.debug('Registering action provider %s' % action_provider)
        
        action_provider = IActionProvider(action_provider)
        
        try:
            # Check if the action provider is not already registered
            action_provider.__ui_merge_id
        except AttributeError:
            assert action_provider.action_group
            self.ui_manager.insert_action_group(action_provider.action_group, -1)
            
            try:
                menu_xml = action_provider.menu_xml
            except AttributeError:
                pass
            else:
                action_provider.__ui_merge_id = \
                        self.ui_manager.add_ui_from_string(menu_xml)

    @component.adapter(ServiceInitializedEvent)
    def _service_initialized_handler(self, event):
        
        self.logger.debug('Handling ServiceInitializedEvent')
        self.logger.debug('Service is %s' % event.service)
        
        if IActionProvider.providedBy(event.service):
            
            self.logger.debug('Loading registered service %s' % event.service)
            
            self.register_action_provider(event.service)


class UIManager(gtk.UIManager):
    """
    Service version of gtk.UIManager.
    """

    interface.implements(IService)


    def init(self, app=None):
        pass

    def shutdown(self):
        pass


# vim:sw=4:et:ai



#!/usr/bin/env python

"""This is Gaphor, a Python+GTK based UML modelling tool.

This module allows Gaphor to be launched from the command line.
The main() function sets up the command-line options and arguments and
passes them to the main Application instance."""

__all__ = [ 'main' ]

from optparse import OptionParser
import logging
import pygtk

from gaphor.application import Application

pygtk.require('2.0')

LOG_FORMAT = '%(name)s %(levelname)s %(message)s'


def launch(model=None):
    """Start the main application by initiating and running Application.
    
    The file_manager service is used here to load a Gaphor model if one was
    specified on the command line.  Otherwise, a new model is created and
    the Gaphor GUI is started."""

    # Make sure gui is loaded ASAP.
    # This prevents menu items from appearing at unwanted places.
    Application.essential_services.append('main_window')

    Application.init()

    main_window = Application.get_service('main_window')

    main_window.open()

    file_manager = Application.get_service('file_manager')

    if model:
        file_manager.load(model)
    else:
        file_manager.action_new()

    Application.run()
    
def main():
    """Start Gaphor from the command line.  This function creates an option
    parser for retrieving arguments and options from the command line.  This
    includes a Gaphor model to load.
    
    The application is then initialized, passing along the option parser.  This
    provides and plugins and services with access to the command line options
    and may add their own."""

    parser = OptionParser()

    parser.add_option('-p',\
                      '--profiler',\
                      action='store_true',\
                      help='Run in profiler')
    parser.add_option('-q', "--quiet",
                      dest='quiet', help='Quiet output',
                      default=False, action='store_true')
    parser.add_option('-v', '--verbose',
                      dest='verbose', help='Verbose output',
                      default=False, action="store_true")
    
    options, args = parser.parse_args()
    
    if options.verbose:
        logging.basicConfig(level=logging.DEBUG, format=LOG_FORMAT)
    elif options.quiet:
        logging.basicConfig(level=logging.WARNING, format=LOG_FORMAT)
    else:
        logging.basicConfig(level=logging.INFO, format=LOG_FORMAT)
    
    try:
        model = args[0]
    except IndexError:
        model = None

    if options.profiler:

        import cProfile
        import pstats

        cProfile.run('import gaphor; gaphor.launch()',\
                     'gaphor.prof')
        
        profile_stats = pstats.Stats('gaphor.prof')
        profile_stats.strip_dirs().sort_stats('time').print_stats(50)

    else:
	
        launch(model)

# TODO: Remove this.  
import __builtin__

__builtin__.__dict__['log'] = logging.getLogger('Gaphor')

# vim:sw=4:et:ai




from unittest import TestCase
from gaphor.services.properties import Properties, FileBackend
import tempfile

#class MockApplication(object):
#
#    def __init__(self):
#        self.events = []
#
#    def handle(self, event):
#        self.events.append(event)
#

class TestProperties(TestCase):

    def setUp(self):
        self.tmpdir = tempfile.mkdtemp()
        backend = FileBackend(self.tmpdir)
        self.properties = Properties(backend)
#        self.app = MockApplication()
        self.properties.init(self.app)

    def shutDown(self):
        self.properties.shutdown()
        os.remove(os.path.join(self.tmpdir, FileBackend.RESOURCE_FILE))
        os.rmdir(self.tmpdir)

#    def test_properties(self):
#        prop = self.properties
#        assert not self.app.events
#
#        prop.set('test1', 2)
#        assert len(self.app.events) == 1, self.app.events
#        event = self.app.events[0]
#        assert 'test1' == event.name
#        assert None is event.old_value
#        assert 2 is event.new_value
#        assert 2 == prop('test1')
#
#        prop.set('test1', 2)
#        assert len(self.app.events) == 1
#
#        prop.set('test1', 'foo')
#        assert len(self.app.events) == 2
#        event = self.app.events[1]
#        assert 'test1' == event.name
#        assert 2 is event.old_value
#        assert 'foo' is event.new_value
#        assert 'foo' == prop('test1')
#
#        assert 3 == prop('test2', 3)
#        assert 3 == prop('test2', 4)


# vim:sw=4:et:ai



# vim: sw=4
"""
Factory for and registration of model elements.
"""

from zope import interface
from zope import component
import uuid
from gaphor.core import inject
from gaphor.misc import odict
from gaphor.interfaces import IService, IEventFilter
from gaphor.UML.interfaces import IElementCreateEvent, IElementDeleteEvent, \
                                  IFlushFactoryEvent, IModelFactoryEvent, \
                                  IElementChangeEvent, IElementEvent
from gaphor.UML.event import ElementCreateEvent, ElementDeleteEvent, \
                             FlushFactoryEvent, ModelFactoryEvent
from gaphor.UML.element import Element
from gaphor.UML.diagram import Diagram


class ElementFactory(object):
    """
    The ElementFactory is used to create elements and do lookups to
    elements.

    Notifications are send with as arguments (name, element, *user_data).
    The following names are used:
    create - a new model element is created (element is newly created element)
    remove - a model element is removed (element is to be removed element)
    model - a new model has been loaded (element is None)
    flush - model is flushed: all element are removed from the factory
            (element is None)
    """
    def __init__(self):
        self._elements = odict.odict()
        self._observers = list()

    def create(self, type):
        """
        Create a new model element of type ``type``.
        """
        obj = self.create_as(type, str(uuid.uuid1()))
        return obj

    def create_as(self, type, id):
        """
        Create a new model element of type 'type' with 'id' as its ID.
        This method should only be used when loading models, since it does
        not emit an ElementCreateEvent event.
        """
        assert issubclass(type, Element)
        obj = type(id, self)
        self._elements[id] = obj
        return obj

    def bind(self, element):
        """
        Bind an already created element to the element factory.
        The element may not be bound to another factory already.
        """
        if hasattr(element, '_factory') and element._factory:
            raise AttributeError, "element is already bound"
        if self._elements.get(element.id):
            raise AttributeError, "an element already exists with the same id"

        element._factory = self
        self._elements[element.id] = element
        

    def size(self):
        """
        Return the amount of elements currently in the factory.
        """
        return len(self._elements)


    def lookup(self, id):
        """
        Find element with a specific id.
        """
        return self._elements.get(id)


    __getitem__ = lookup


    def __contains__(self, element):
        return self.lookup(element.id) is element


    def select(self, expression=None):
        """
        Iterate elements that comply with expression.
        """
        if expression is None:
            for e in self._elements.itervalues():
                yield e
        else:
            for e in self._elements.itervalues():
                if expression(e):
                    yield e


    def lselect(self, expression=None):
        """
        Like select(), but returns a list.
        """
        return list(self.select(expression))


    def keys(self):
        """
        Return a list with all id's in the factory.
        """
        return self._elements.keys()


    def iterkeys(self):
        """
        Return a iterator with all id's in the factory.
        """
        return self._elements.iterkeys()


    def values(self):
        """
        Return a list with all elements in the factory.
        """
        return self._elements.values()


    def itervalues(self):
        """
        Return a iterator with all elements in the factory.
        """
        return self._elements.itervalues()


    def is_empty(self):
        """
        Returns True if the factory holds no elements.
        """
        return bool(self._elements)


    def flush(self):
        """Flush all elements (remove them from the factory). 
        
        Diagram elements are flushed first.  This is so that canvas updates
        are blocked.  The remaining elements are then flushed.
        """
        
        flush_element = self._flush_element
        for element in self.lselect(lambda e: isinstance(e, Diagram)):
            element.canvas.block_updates = True
            flush_element(element)
                
        for element in self.lselect():
            flush_element(element)

    def _flush_element(self, element):
        element.unlink()

    def _unlink_element(self, element):
        """
        NOTE: Invoked from Element.unlink() to perform an element unlink.
        """
        try:
            del self._elements[element.id]
        except KeyError:
            pass

    def swap_element(self, element, new_class):
	assert element in self._elements.values()
        if element.__class__ is not new_class:
            element.__class__ = new_class

    def _handle(self, event):
        """
        Handle events coming from elements.
        """
        # Invoke default handler, so properties get updated.
        component.handle(event)


class ElementFactoryService(ElementFactory):
    """
    Service version of the ElementFctory.
    """
    interface.implements(IService)

    component_registry = inject('component_registry')

    def init(self, app):
        pass

    def shutdown(self):
        self.flush()

    def create(self, type):
        """
        Create a new model element of type ``type``.
        """
        obj = super(ElementFactoryService, self).create(type)
        self.component_registry.handle(ElementCreateEvent(self, obj))
        return obj

    def flush(self):
        """Flush all elements (remove them from the factory).  First test
        if the element factory has a Gaphor application instance.  If yes,
        the application will handle a FlushFactoryEvent and will register
        a ElementChangedEventBlocker adapter.
        
        Diagram elements are flushed first.  This is so that canvas updates
        are blocked.  The remaining elements are then flushed.  Finally,
        the ElementChangedEventBlocker adapter is unregistered if the factory
        has an application instance."""
        
        self.component_registry.handle(FlushFactoryEvent(self))
        self.component_registry.register_subscription_adapter(ElementChangedEventBlocker)

        try:
            super(ElementFactoryService, self).flush()
        finally:
            self.component_registry.unregister_subscription_adapter(ElementChangedEventBlocker)

    def notify_model(self):
        """
        Send notification that a new model has been loaded by means of the
        ModelFactoryEvent event from gaphor.UML.event.
        """
        self.component_registry.handle(ModelFactoryEvent(self))

    def _unlink_element(self, element):
        """
        NOTE: Invoked from Element.unlink() to perform an element unlink.
        """
        self.component_registry.handle(ElementDeleteEvent(self, element))
        super(ElementFactoryService, self)._unlink_element(element)

    def _handle(self, event):
        """
        Handle events coming from elements (used internally).
        """
        self.component_registry.handle(event)


class ElementChangedEventBlocker(object):
    """
    Blocks all events of type IElementChangeEvent.

    This filter is placed when the the element factory flushes it's content.
    """
    component.adapts(IElementChangeEvent)
    interface.implements(IEventFilter)

    def __init__(self, event):
        self._event = event

    def filter(self):
        """
        Returns something that evaluates to `True` so events are blocked.
        """
        return 'Blocked by ElementFactory.flush()'



# vim:sw=4:et



# -*- coding: utf-8 -*-
# vim:sw=4:et:ai

# Copyright © 2010 etk.docking Contributors
#
# This file is part of etk.docking.
#
# etk.docking is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# etk.docking is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with etk.docking. If not, see <http://www.gnu.org/licenses/>.


from __future__ import absolute_import
import sys

from simplegeneric import generic
from xml.etree.ElementTree import Element, SubElement, tostring, fromstring

import gtk

from etk.docking import DockFrame, DockPaned, DockGroup, DockItem
from gaphor.core import _

SERIALIZABLE = ( DockFrame, DockPaned, DockGroup, DockItem )


def serialize(layout):
    def _ser(widget, element):
        if isinstance(widget, SERIALIZABLE):
            sub = SubElement(element, type(widget).__name__.lower() , attributes(widget))
            widget.foreach(_ser, sub)
        else:
            sub = SubElement(element, 'widget', attributes(widget))

    tree = Element('layout')
    map(_ser, layout.frames, [tree] * len(layout.frames))

    return tostring(tree, encoding=sys.getdefaultencoding())

widget_factory = {}

def deserialize(layout, container, layoutstr, itemfactory):
    '''
    Return a new layout with it's attached frames. Frames that should be floating
    already have their gtk.Window attached (check frame.get_parent()). Transient settings
    and such should be done by the invoking application.
    '''
    def _des(element, parent_widget=None):
        if element.tag == 'widget':
            name = element.attrib['name']
            widget = itemfactory(name)
            widget.set_name(name)
            parent_widget.add(widget)
        else:
            factory = widget_factory[element.tag]
            widget = factory(parent=parent_widget, **element.attrib)
            assert widget, 'No widget (%s)' % widget
            if len(element):
                map(_des, element, [widget] * len(element))
        return widget

    tree = fromstring(layoutstr)
    map(layout.add, map(_des, tree, [ container ] * len(tree)))

    return layout

def parent_attributes(widget):
    """
    Add properties defined in the parent widget specific for this instance (like weight).
    """
    container = widget.get_parent()
    d = {}

    if isinstance(container, DockPaned):
        paned_item = [i for i in container._items if i.child is widget][0]
        if paned_item.weight:
            d['weight'] = str(int(paned_item.weight * 100))

    return d

@generic
def attributes(widget):
    raise NotImplementedError

@attributes.when_type(gtk.Widget)
def widget_attributes(widget):
    return { 'name': widget.get_name() or 'empty' }

@attributes.when_type(DockItem)
def dock_item_attributes(widget):
    d = { 'title': widget.props.title,
             'tooltip': widget.props.title_tooltip_text }
    if widget.props.icon_name:
        d['icon_name'] = widget.props.icon_name
    if widget.props.stock:
        d['stock_id'] = widget.props.stock
    return d

@attributes.when_type(DockGroup)
def dock_group_attributes(widget):
    d = parent_attributes(widget)
    name = widget.get_name()
    if name != widget.__gtype__.name:
        d['name'] = name
    return d

@attributes.when_type(DockPaned)
def dock_paned_attributes(widget):
    return dict(orientation=(widget.get_orientation() == gtk.ORIENTATION_HORIZONTAL and 'horizontal' or 'vertical'),
                **parent_attributes(widget))

@attributes.when_type(DockFrame)
def dock_frame_attributes(widget):
    a = widget.allocation
    d = dict(width=str(a.width), height=str(a.height))
    parent = widget.get_parent()

    if isinstance(parent, gtk.Window) and parent.get_transient_for():
        d['floating'] = 'true'
        d['x'], d['y'] = map(str, parent.get_position())

    return d

def factory(typename):
    '''
    Simple decorator for populating the widget_factory dictionary.
    '''
    def _factory(func):
        widget_factory[typename] = func
        return func

    return _factory

@factory('dockitem')
def dock_item_factory(parent, title, tooltip, icon_name=None, stock_id=None, pos=None, vispos=None, current=None, name=None):
    item = DockItem(_(title), _(tooltip), icon_name, stock_id)
    if name:
        item.set_name(name)
    if pos:
        pos = int(pos)
    if vispos:
        vispos = int(vispos)
    parent.insert_item(item, pos, vispos)

    item.show()

    return item

@factory('dockgroup')
def dock_group_factory(parent, weight=None, name=None):
    group = DockGroup()

    if name:
        group.set_name(name)

    if weight is not None:
        parent.insert_item(group, weight=float(weight) / 100.)
    else:
        parent.add(group)

    group.show()

    return group

@factory('dockpaned')
def dock_paned_factory(parent, orientation, weight=None, name=None):
    paned = DockPaned()

    if name:
        paned.set_name(name)

    if orientation == 'horizontal':
        paned.set_orientation(gtk.ORIENTATION_HORIZONTAL)
    else:
        paned.set_orientation(gtk.ORIENTATION_VERTICAL)

    if weight is not None:
        item = parent.insert_item(paned, weight=float(weight) / 100.)
    else:
        parent.add(paned)

    paned.show()

    return paned

@factory('dockframe')
def dock_frame_factory(parent, width, height, floating=None, x=None, y=None):
    frame = DockFrame()
    frame.set_size_request(int(width), int(height))

    if floating == 'true':
        window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        #self.window.set_type_hint(gdk.WINDOW_TYPE_HINT_UTILITY)
        window.set_property('skip-taskbar-hint', True)
        window.move(int(x), int(y))
        window.add(frame)
        window.set_transient_for(parent)
        window.show()
    else:
        parent.add(frame)
    
    frame.show()

    return frame



"""Application service test cases."""

import unittest

from zope import component

from gaphor import UML
from gaphor.application import Application
from gaphor.interfaces import IService

class LoadServiceTestCase(unittest.TestCase):

    """Test case for loading Gaphor services."""

    def test_service_load(self):
        """Test loading services and querying utilities."""
        
        Application.init(['undo_manager', 'file_manager', 'properties'])

        self.assertTrue(Application.get_service('undo_manager') is not None,\
                        'Failed to load the undo manager service')
                        
        self.assertTrue(Application.get_service('file_manager') is not None,\
                        'Failed to load the file manager service')

        self.assertTrue(component.queryUtility(IService, 'undo_manager') is not None,\
                        'Failed to query the undo manager utility')
                        
        self.assertTrue(component.queryUtility(IService, 'file_manager') is not None,\
                        'Failed to query the file manager utility')

        Application.shutdown()



"""
A version of the standard gtk.ColorButton tweaked towards Gaphor.

Gaphor is using color values from 0 to 1 (cairo standard), so that required some tweaks
on the color widget. The standard format is `(red, green, blue, alpha)`.
"""

import gtk


class ColorButton(gtk.ColorButton):

    __gtype_name__ = 'GaphorColorButton'

    def __init__(self, r, g, b, a):
        gtk.Button.__init__(self)
        self.set_color(gtk.gdk.Color(int(r * 65535),
                                     int(g * 65535),
                                     int(b * 65535)))
        self.set_use_alpha(True)
        self.set_alpha(int(a * 65535))

    def get_color_float(self):
        c = self.get_color()
        return (c.red_float, c.green_float, c.blue_float, self.get_alpha() / 65535.)

    color = property(lambda s: s.get_color_float())

# vim:sw=4:et:ai




from gaphor.ui.elementeditor import ElementEditor
from gaphor.tests.testcase import TestCase

class ElementEditorTestCase(TestCase):

    services = TestCase.services + ['main_window', 'ui_manager', 'action_manager', 'properties']

    def test1(self):
        import gtk
        window = ElementEditor()
        assert len(window.action_group.list_actions()) == 1, window.action_group.list_actions()
        window.open()
        window.close()




"""
Transation support for Gaphor
"""

from logging import getLogger
from zope import interface, component
from gaphor.interfaces import ITransaction
from gaphor.event import TransactionBegin, TransactionCommit, TransactionRollback
from gaphor import application

logger = getLogger('transaction')

def transactional(func):
    """The transactional decorator makes a function transactional.  A
    Transaction instance is created before the decorated function is called.
    If calling the function leads to an exception being raised, the transaction
    is rolled-back.  Otherwise, it is committed."""
    
    def _transactional(*args, **kwargs):
        r = None
        tx = Transaction()
        try:
            r = func(*args, **kwargs)
        except Exception, e:
            log.error('Transaction terminated due to an exception, performing a rollback', exc_info=True)
            try:
                tx.rollback()
            except Exception, e:
                log.error('Rollback failed', exc_info=True)
            raise
        else:
            tx.commit()
        return r
    return _transactional

class TransactionError(Exception):
    """
    Errors related to the transaction module.
    """

class Transaction(object):
    """
    The transaction. On start and end of a transaction an event is emited.

    Transactions can be nested. If the outermost transaction is committed or
    rolled back, an event is emitted.

    Events can be handled programmatically:

    >>> tx = Transaction()
    >>> tx.commit()

    It can be assigned as decorator:

    >>> @transactional
    ... def foo():
    ...     pass

    Or with the ``with`` statement:

    >>> with Transaction():
    ...     pass
    """

    interface.implements(ITransaction)

    component_registry = application.inject('component_registry')

    _stack= []

    def __init__(self):
        """Initialize the transaction.  If this is the first transaction in
        the stack, a TransactionBegin event is emited."""
        
        self._need_rollback = False
        if not self._stack:
            self._handle(TransactionBegin())
        self._stack.append(self)

    def commit(self):
        """Commit the transaction.  First, the transaction is closed.
        If it needs to be rolled-back, a TransactionRollback event is emited.
        Otherwise, a TransactionCommit event is emited."""
        
        self._close()
        if not self._stack:
            if self._need_rollback:
                self._handle(TransactionRollback())
            else:
                self._handle(TransactionCommit())

    def rollback(self):
        """Roll-back the transaction.  First, the transaction is closed.
        Every transaction on the stack is then marked for roll-back.  If
        the stack is empty, a TransactionRollback event is emited."""
        
        self._close()
        for tx in self._stack:
            tx._need_rollback = True
        else:
            if not self._stack:
                self._handle(TransactionRollback())

    def _close(self):
        """Close the transaction.  If the stack is empty, a TransactionError
        is raised.  If the last transaction on the stack isn't this transaction,
        a Transaction error is raised."""
        
        try:
            last = self._stack.pop()
        except IndexError:
            raise TransactionError, 'No Transaction on stack.'
        if last is not self:
            self._stack.append(last)
            raise TransactionError, 'Transaction on stack is not the transaction being closed.'

    def _handle(self, event):
        try:
            component_registry = self.component_registry
        except (application.NotInitializedError, component.ComponentLookupError):
            logger.warning('Could not lookup component_registry. Not emiting events.')
        else:
            component_registry.handle(event)

    def __enter__(self):
        """Provide with-statement transaction support."""
        return self

    def __exit__(self, exc_type=None, exc_val=None, exc_tb=None):
        """Provide with-statement transaction support.  If an error occured,
        the transaction is rolled back.  Otherwise, it is committed."""
        
        if exc_type:
            self.rollback()
        else:
            self.commit()

# vim: sw=4:et:ai




from gaphor.tests import TestCase
from gaphor import UML
from gaphor.diagram import items
from gaphor.diagram.interfaces import IConnect
from zope import component
from gaphas.aspect import ConnectionSink
import gaphor.adapters.classes.classconnect

class GaphasTest(TestCase):

    services = TestCase.services + [ 'sanitizer_service', 'undo_manager' ]

    def test_remove_class_with_association(self):
        c1 = self.create(items.ClassItem, UML.Class)
        c1.name = 'klassitem1'
        c2 = self.create(items.ClassItem, UML.Class)
        c2.name = 'klassitem2'

        a = self.create(items.AssociationItem)

        assert 3 == len(self.diagram.canvas.get_all_items())

        self.connect(a, a.head, c1)
        self.connect(a, a.tail, c2)

        assert a.subject
        assert self.element_factory.lselect(lambda e: e.isKindOf(UML.Association))[0] is a.subject
        
        c1.unlink()

        self.diagram.canvas.update_now()

# vim:sw=4:et:ai





import unittest
from gaphor import UML
from gaphor.application import Application
import pkg_resources

class PackageWithStereotypesRemovalTestCase(unittest.TestCase):

    def setUp(self):
        Application.init()
        element_factory = Application.get_service('element_factory')
        from gaphor.storage.storage import load
        load('tests/issue_53.gaphor', element_factory)

    def tearDown(self):
        Application.get_service('element_factory').shutdown()
        Application.shutdown()

    def testPackageRemoval(self):
        # Load the application
        element_factory = Application.get_service('element_factory')

        # Find all profile instances
        profiles = element_factory.lselect(lambda e: e.isKindOf(UML.Profile))
        
        # Check there is 1 profile
        self.assertEquals(1, len(profiles))

        # Check the profile has 1 presentation
        self.assertEquals(1, len(profiles[0].presentation))

        # Unlink the presentation
        profiles[0].presentation[0].unlink()

        self.assertFalse(element_factory.lselect(lambda e: e.isKindOf(UML.Profile)))

        classes = element_factory.lselect(lambda e: e.isKindOf(UML.Class))
        self.assertEquals(1, len(classes))

        # Check if the link is really removed:
        self.assertFalse(classes[0].appliedStereotype)
        self.assertFalse(element_factory.lselect(lambda e: e.isKindOf(UML.InstanceSpecification)))
        self.assertEquals(3, len(element_factory.lselect(lambda e: e.isKindOf(UML.Diagram))))

    def testPackageRemovalByRemovingTheDiagram(self):
        element_factory = Application.get_service('element_factory')

        diagram = element_factory.lselect(lambda e: e.isKindOf(UML.Diagram) and e.name == 'Stereotypes diagram')[0]
        
        self.assertTrue(diagram)

        diagram.unlink()

        self.assertFalse(element_factory.lselect(lambda e: e.isKindOf(UML.Profile)))

        classes = element_factory.lselect(lambda e: e.isKindOf(UML.Class))
        self.assertEquals(1, len(classes))

        # Check if the link is really removed:
        self.assertFalse(classes[0].appliedStereotype)
        self.assertFalse(element_factory.lselect(lambda e: e.isKindOf(UML.InstanceSpecification)))
        self.assertEquals(2, len(element_factory.lselect(lambda e: e.isKindOf(UML.Diagram))))

# vim:sw=4:et:ai



#!/usr/bin/env python

import gtk
from cairo import Matrix
from zope import component
from etk.docking import DockItem

from gaphas.view import GtkView
from gaphas.painter import PainterChain, ItemPainter, HandlePainter, \
                           FocusedItemPainter, ToolPainter, BoundingBoxPainter
from gaphas.freehand import FreeHandPainter
from gaphas import segment, guide

from gaphor import UML
from gaphor.core import _, inject, transactional, action, toggle_action, build_action_group
from gaphor.UML.interfaces import IAttributeChangeEvent, IElementDeleteEvent
from gaphor.diagram import get_diagram_item
from gaphor.diagram.items import DiagramItem
from gaphor.transaction import Transaction
from gaphor.ui.diagramtoolbox import DiagramToolbox
from gaphor.ui.event import DiagramSelectionChange
from gaphor.services.properties import IPropertyChangeEvent

class DiagramTab(object):
    
    component_registry = inject('component_registry')
    element_factory = inject('element_factory')
    action_manager = inject('action_manager')

    menu_xml = """
      <ui>
        <menubar action="mainwindow">
          <menu action="edit">
            <placeholder name="ternary">
              <menuitem action="diagram-delete" />
              <separator />
              <menuitem action="diagram-select-all" />
              <menuitem action="diagram-unselect-all" />
              <separator />
            </placeholder>
          </menu>
          <menu action="diagram">
            <placeholder name="secondary">
              <menuitem action="diagram-zoom-in" />
              <menuitem action="diagram-zoom-out" />
              <menuitem action="diagram-zoom-100" />
              <separator />
              <menuitem action="diagram-close" />
            </placeholder>
          </menu>
        </menubar>
        <toolbar name='mainwindow-toolbar'>
          <placeholder name="left">
            <separator />
            <toolitem action="diagram-zoom-in" />
            <toolitem action="diagram-zoom-out" />
            <toolitem action="diagram-zoom-100" />
          </placeholder>
        </toolbar>
      </ui>
    """

    VIEW_TARGET_STRING = 0
    VIEW_TARGET_ELEMENT_ID = 1
    VIEW_TARGET_TOOLBOX_ACTION = 2
    VIEW_DND_TARGETS = [
        ('gaphor/element-id', 0, VIEW_TARGET_ELEMENT_ID),
        ('gaphor/toolbox-action', 0, VIEW_TARGET_TOOLBOX_ACTION)]


    def __init__(self, diagram):
        self.diagram = diagram
        self.view = None
        #self.owning_window = owning_window
        self.action_group = build_action_group(self)
        self.toolbox = None
        self.component_registry.register_handler(self._on_element_change)
        self.component_registry.register_handler(self._on_element_delete)

    title = property(lambda s: s.diagram and s.diagram.name or _('<None>'))

    def get_diagram(self):
        return self.diagram

    def get_view(self):
        return self.view

    def get_canvas(self):
        return self.diagram.canvas

    def construct(self):
        """
        Create the widget.
        
        Returns: the newly created widget, a DockItem.
        """
        assert self.diagram

        view = GtkView(canvas=self.diagram.canvas)
        view.drag_dest_set(gtk.DEST_DEFAULT_MOTION, DiagramTab.VIEW_DND_TARGETS,
                           gtk.gdk.ACTION_MOVE | gtk.gdk.ACTION_COPY | gtk.gdk.ACTION_LINK)

        scrolled_window = gtk.ScrolledWindow()
        scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolled_window.set_shadow_type(gtk.SHADOW_IN)
        scrolled_window.add(view)
        scrolled_window.show_all()

        view.connect('focus-changed', self._on_view_selection_changed)
        view.connect('selection-changed', self._on_view_selection_changed)
        view.connect_after('key-press-event', self._on_key_press_event)
        view.connect('drag-drop', self._on_drag_drop)
        view.connect('drag-data-received', self._on_drag_data_received)

        self.view = view
        
        self.toolbox = DiagramToolbox(self.diagram, view)
        
        item = DockItem(title=self.title, stock_id='gaphor-diagram')
        item.add(scrolled_window)

        self.widget = item

        return item


    @component.adapter(IAttributeChangeEvent)
    def _on_element_change(self, event):
        if event.element is self.diagram and \
                event.property is UML.Diagram.name:
           self.widget.title = self.title


    @component.adapter(IElementDeleteEvent)
    def _on_element_delete(self, event):
        if event.element is self.diagram:
            self.close()


    @action(name='diagram-close', stock_id='gtk-close')
    def close(self):
        """
        Tab is destroyed. Do the same thing that would
        be done if File->Close was pressed.
        """
        self.widget.destroy()
        self.component_registry.unregister_handler(self._on_element_delete)
        self.component_registry.unregister_handler(self._on_element_change)
        self.view = None


    @action(name='diagram-zoom-in', stock_id='gtk-zoom-in')
    def zoom_in(self):
        self.view.zoom(1.2)


    @action(name='diagram-zoom-out', stock_id='gtk-zoom-out')
    def zoom_out(self):
        self.view.zoom(1 / 1.2)


    @action(name='diagram-zoom-100', stock_id='gtk-zoom-100')
    def zoom_100(self):
        zx = self.view.matrix[0]
        self.view.zoom(1 / zx)


    @action(name='diagram-select-all', label='_Select all', accel='<Control>a')
    def select_all(self):
        self.view.select_all()


    @action(name='diagram-unselect-all', label='Des_elect all',
            accel='<Control><Shift>a')
    def unselect_all(self):
        self.view.unselect_all()

        
    @action(name='diagram-delete', stock_id='gtk-delete')
    @transactional
    def delete_selected_items(self):
        items = self.view.selected_items
        for i in list(items):
            if isinstance(i, DiagramItem):
                i.unlink()
            else:
                if i.canvas:
                    i.canvas.remove(i)


    def set_drawing_style(self, sloppiness=0.0):
        """Set the drawing style for the diagram. 0.0 is straight, 
        2.0 is very sloppy.  If the sloppiness is set to be anything
        greater than 0.0, the FreeHandPainter instances will be used
        for both the item painter and the box painter.  Otherwise, by
        default, the ItemPainter is used for the item and 
        BoundingBoxPainter for the box."""

        view = self.view
        
        if sloppiness:
            
            item_painter = FreeHandPainter(ItemPainter(),\
                                           sloppiness=sloppiness)
            box_painter = FreeHandPainter(BoundingBoxPainter(),\
                                          sloppiness=sloppiness)
        
        else:
        
            item_painter = ItemPainter()
            box_painter = BoundingBoxPainter()
            
        view.painter = PainterChain().\
                       append(item_painter).\
                       append(HandlePainter()).\
                       append(FocusedItemPainter()).\
                       append(ToolPainter())
                       
        view.bounding_box_painter = box_painter
        
        view.queue_draw_refresh()

    def may_remove_from_model(self, view):
        """
        Check if there are items which will be deleted from the model
        (when their last views are deleted). If so request user
        confirmation before deletion.
        """
        items = self.view.selected_items
        last_in_model = filter(lambda i: i.subject and len(i.subject.presentation) == 1, items)
        log.debug('Last in model: %s' % str(last_in_model))
        if last_in_model:
            return self.confirm_deletion_of_items(last_in_model)
        return True


    def confirm_deletion_of_items(self, last_in_model):
        """
        Request user confirmation on deleting the item from the model.
        """
        s = ''
        for item in last_in_model:
            s += '%s\n' % str(item)

        dialog = gtk.MessageDialog(
                None,
                gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                gtk.MESSAGE_WARNING,
                gtk.BUTTONS_YES_NO,
                'This will remove the following selected items from the model:\n%s\nAre you sure?' % s
                )
        dialog.set_transient_for(self.get_toplevel())
        value = dialog.run()
        dialog.destroy()
        if value == gtk.RESPONSE_YES:
            return True
        return False


    def _on_key_press_event(self, view, event):
        """
        Handle the 'Delete' key. This can not be handled directly (through
        GTK's accelerators) since otherwise this key will confuse the text
        edit stuff.
        """
        if view.is_focus():
            if event.keyval == gtk.keysyms.Delete and \
                    (event.state == 0 or event.state & gtk.gdk.MOD2_MASK):
                self.delete_selected_items()
            elif event.keyval == gtk.keysyms.BackSpace and \
                    (event.state == 0 or event.state & gtk.gdk.MOD2_MASK):
                self.delete_selected_items()


    def _on_view_selection_changed(self, view, selection_or_focus):
        self.component_registry.handle(DiagramSelectionChange(view, view.focused_item, view.selected_items))


    def _on_drag_drop(self, view, context, x, y, time):
        print 'drag_drop on', context.targets
        if self.VIEW_DND_TARGETS[0][0] in context.targets:
            target = gtk.gdk.atom_intern(self.VIEW_DND_TARGETS[0][0])
            view.drag_get_data(context, target, time)
            return True
        elif self.VIEW_DND_TARGETS[1][0] in context.targets:
            target = gtk.gdk.atom_intern(self.VIEW_DND_TARGETS[1][0])
            view.drag_get_data(context, target, time)
            return True
        return False

    def _on_drag_data_received(self, view, context, x, y, data, info, time):
        """
        Handle data dropped on the canvas.
        """
        print 'DND data received', view
        if data and data.format == 8 and info == DiagramTab.VIEW_TARGET_TOOLBOX_ACTION:
            tool = self.toolbox.get_tool(data.data)
            tool.create_item((x, y))
            context.finish(True, False, time)
        elif data and data.format == 8 and info == DiagramTab.VIEW_TARGET_ELEMENT_ID:
            #print 'drag_data_received:', data.data, info
            n, p = data.data.split('#')
            element = self.element_factory.lookup(n)
            assert element

            # TODO: use adapters to execute code below

            q = type(element)
            if p:
                q = q, p
            item_class = get_diagram_item(q)
            if isinstance(element, UML.Diagram):
                self.action_manager.execute('OpenModelElement')
            elif item_class:
                tx = Transaction()
                item = self.diagram.create(item_class)
                assert item
                
                x, y = view.get_matrix_v2i(item).transform_point(x, y)
                item.matrix.translate(x, y)
                item.subject = element
                tx.commit()
                view.unselect_all()
                view.focused_item = item

            else:
                log.warning('No graphical representation for UML element %s' % type(element).__name__)
            context.finish(True, False, time)
        #else:
        #    context.finish(False, False, time)

# vim: sw=4:et:ai



"""
This module contains user interface related code, such as the
main screen and diagram windows.
"""

import gtk
import pkg_resources
import os.path

icon_theme = gtk.icon_theme_get_default()
icon_theme.append_search_path(os.path.abspath(
                                pkg_resources.resource_filename('gaphor.ui', 'pixmaps')))

import re
def _repl(m):
    v = m.group(1).lower()
    return len(v) == 1 and v or '%c-%c' % tuple(v)


_repl.expr = '(.?[A-Z])'


def icon_for_element(element):
    return re.sub(_repl.expr, _repl, type(element).__name__)

# vim:sw=4:et:




"""Defines a QuestionDialog class used to get a yes or no answer from the user.
"""

import gtk

class QuestionDialog(object):
    """A dialog that displays a GTK MessageDialog to get a yes or no answer
    from the user."""
    
    def __init__(self, question, parent=None):
        """Create the QuestionDialog.  The question parameter is a question
        string to ask the user.  The parent parameter is the parent window
        of the dialog."""
        
        self.dialog = gtk.MessageDialog(parent,\
                                        gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,\
                                        gtk.MESSAGE_QUESTION,\
                                        gtk.BUTTONS_YES_NO,\
                                        question)
                                        
    def get_answer(self):
        """Return answer to the question by running the dialog.  The answer
        is accessed via the answer attribute."""
        
        answer = self.dialog.run()
        
        if answer == gtk.RESPONSE_YES:
            return True
            
        return False
        
    def destroy(self):
        """Destroy the GTK dialog."""
        
        self.dialog.destroy()

    answer = property(get_answer)




"""Provides a get_user_data_dir() function for retrieving the user's data
directory."""

import os

def get_user_data_dir():
    
    """Return the directory where the user's data is stored.  This varies
    depending on platform."""
    
    if os.name == 'nt':
        home = 'USERPROFILE'
    else:
        home = 'HOME'

    return os.path.join(os.getenv(home), '.gaphor')




from gaphor.misc.xmlwriter import XMLWriter

class XMIExport(object):
    
    XMI_VERSION = '2.1'
    XMI_NAMESPACE = 'http://schema.omg.org/spec/XMI/2.1'
    UML_NAMESPACE = 'http://schema.omg.org/spec/UML/2.1'
    XMI_PREFIX = 'XMI'
    UML_PREFIX = 'UML'
    
    def __init__(self, element_factory):
        self.element_factory = element_factory
        self.handled_ids = list()
        
    def handle(self, xmi, element):
        log.debug('Handling %s'%element.__class__.__name__)
        try:
            handler_name = 'handle%s'%element.__class__.__name__
            handler = getattr(self, handler_name)
            idref = element.id in self.handled_ids
            handler(xmi, element, idref=idref)
            if not idref:
                self.handled_ids.append(element.id)
        except AttributeError, e:
            log.warning('Missing handler for %s:%s'%(element.__class__.__name__,e))
        except Exception, e:
            log.error('Failed to handle %s:%s'%(element.__class__.__name__, e))
            
    def handlePackage(self, xmi, element, idref=False):
        
        attributes = dict()
        attributes['%s:id'%self.XMI_PREFIX] = element.id
        attributes['name'] = element.name
        attributes['visibility'] = element.visibility

        xmi.startElement('%s:Package'%self.UML_PREFIX, attrs=attributes)
                         
        for ownedMember in element.ownedMember:
            xmi.startElement('ownedMember', attrs=dict())
            self.handle(xmi, ownedMember)
            xmi.endElement('ownedMember')

        xmi.endElement('%s:Package'%self.UML_PREFIX)

    def handleClass(self, xmi, element, idref=False):
        
        attributes = dict()
        
        if idref:
            attributes['%s:idref'%self.XMI_PREFIX] = element.id
        else:
            attributes['%s:id'%self.XMI_PREFIX] = element.id
            attributes['name'] = element.name
            attributes['isAbstract'] = str(element.isAbstract)
        
        xmi.startElement('%s:Class'%self.UML_PREFIX, attrs=attributes)
        
        if not idref:
        
            for ownedAttribute in element.ownedAttribute:
                xmi.startElement('ownedAttribute', attrs=dict())
                self.handle(xmi, ownedAttribute)
                xmi.endElement('ownedAttribute')
                
            for ownedOperation in element.ownedOperation:
                xmi.startElement('ownedOperation', attrs=dict())
                self.handle(xmi, ownedOperation)
                xmi.endElement('ownedOperation')
            
        xmi.endElement('%s:Class'%self.UML_PREFIX)
        
    def handleProperty(self, xmi, element, idref=False):
        
        attributes = dict()
        attributes['%s:id'%self.XMI_PREFIX] = element.id
        attributes['isStatic'] = str(element.isStatic)
        attributes['isOrdered'] = str(element.isOrdered)
        attributes['isUnique'] = str(element.isUnique)
        attributes['isDerived'] = str(element.isDerived)
        attributes['isDerivedUnion'] = str(element.isDerivedUnion)
        attributes['isReadOnly'] = str(element.isReadOnly)                
        
        if element.name is not None:
            attributes['name'] = element.name
        
        xmi.startElement('%s:Property'%self.UML_PREFIX, attrs=attributes)
        
        #TODO: This should be type, not typeValue.
        if element.typeValue is not None:
            xmi.startElement('type', attrs=dict())
            self.handle(xmi, element.typeValue)
            xmi.endElement('type')
                    
        xmi.endElement('%s:Property'%self.UML_PREFIX)
        
    def handleOperation(self, xmi, element, idref=False):
        
        attributes = dict()
        attributes['%s:id'%self.XMI_PREFIX] = element.id
        attributes['isStatic'] = str(element.isStatic)
        attributes['isQuery'] = str(element.isQuery)
        attributes['name'] = element.name
        
        xmi.startElement('%s:Operation'%self.XMI_PREFIX, attrs=attributes)
        
        for ownedParameter in element.parameter:
            xmi.startElement('ownedElement', attrs=dict())
            self.handle(xmi, ownedParameter)
            xmi.endElement('ownedElement')
        
        xmi.endElement('%s:Operation'%self.XMI_PREFIX)
        
    def handleParameter(self, xmi, element, idref=False):
        
        attributes = dict()
        attributes['%s:id'%self.XMI_PREFIX] = element.id
        attributes['isOrdered'] = str(element.isOrdered)
        attributes['isUnique'] = str(element.isUnique)

        attributes['direction'] = element.direction
        attributes['name'] = element.name
        
        xmi.startElement('%s:Parameter'%self.XMI_PREFIX, attrs=attributes)
        
        xmi.endElement('%s:Parameter'%self.XMI_PREFIX)
        
    def handleLiteralSpecification(self, xmi, element, idref=False):
        
        attributes = dict()
        attributes['%s:id'%self.XMI_PREFIX] = element.id
        attributes['value'] = element.value
        
        xmi.startElement('%s:LiteralSpecification'%self.UML_PREFIX, attrs=attributes)
        
        xmi.endElement('%s:LiteralSpecification'%self.UML_PREFIX)
        
    def handleAssociation(self, xmi, element, idref=False):
        
        attributes = dict()
        attributes['%s:id'%self.XMI_PREFIX] = element.id
        attributes['isDerived'] = str(element.isDerived)
        
        xmi.startElement('%s:Association'%self.UML_PREFIX, attrs=attributes)
        
        for memberEnd in element.memberEnd:
            xmi.startElement('memberEnd', attrs=dict())
            self.handle(xmi, memberEnd)
            xmi.endElement('memberEnd')
            
        for ownedEnd in element.ownedEnd:
            xmi.startElement('ownedEnd', attrs=dict())
            self.handle(xmi, ownedEnd)
            xmi.endElement('ownedEnd')
        
        xmi.endElement('%s:Association'%self.UML_PREFIX)
        
    def handleDependency(self, xmi, element, idref=False):
        
        attributes = dict()
        attributes['%s:id'%self.XMI_PREFIX] = element.id
        
        xmi.startElement('%s:Dependency'%self.UML_PREFIX, attrs=attributes)
        
        for client in element.client:
            xmi.startElement('client', attrs=dict())
            self.handle(xmi, client)
            xmi.endElement('client')
            
        for supplier in element.supplier:
            xmi.startElement('supplier', attrs=dict())
            self.handle(xmi, supplier)
            xmi.endElement('supplier')
        
        xmi.endElement('%s:Dependency'%self.UML_PREFIX)
        
    def handleGeneralization(self, xmi, element, idref=False):
        
        attributes = dict()
        attributes['%s:id'%self.XMI_PREFIX] = element.id
        attributes['isSubstitutable'] = str(element.isSubstitutable)
        
        xmi.startElement('%s:Generalization'%self.UML_PREFIX, attrs=attributes)
        
        if element.general:
            xmi.startElement('general', attrs=dict())
            self.handle(xmi, element.general)
            xmi.endElement('general')
            
        if element.specific:
            xmi.startElement('specific', attrs=dict())
            self.handle(xmi, element.specific)
            xmi.endElement('specific')
        
        xmi.endElement('%s:Generalization'%self.UML_PREFIX)
        
    def handleRealization(self, xmi, element, idref=False):
        
        attributes = dict()
        attributes['%s:id'%self.XMI_PREFIX] = element.id
        
        xmi.startElement('%s:Realization'%self.UML_PREFIX, attrs=attributes)
        
        for client in element.client:
            xmi.startElement('client', attrs=dict())
            self.handle(xmi, client)
            xmi.endElement('client')
            
        for supplier in element.supplier:
            xmi.startElement('supplier', attrs=dict())
            self.handle(xmi, supplier)
            xmi.endElement('supplier')
        
        xmi.endElement('%s:Realization'%self.UML_PREFIX)        
        
    def handleInterface(self, xmi, element, idref=False):
        
        attributes = dict()
        attributes['%s:id'%self.XMI_PREFIX] = element.id
        
        xmi.startElement('%s:Interface'%self.UML_PREFIX, attrs=attributes)
        
        for ownedAttribute in element.ownedAttribute:
            xmi.startElement('ownedAttribute', attrs=dict())
            self.handle(ownedAttribute)
            xmi.endElement('ownedAttribute')
            
        for ownedOperation in element.ownedOperation:
            xmi.startElement('ownedOperation', attrs=dict())
            self.handle(ownedOperation)
            xmi.endElement('ownedOperation')
        
        xmi.endElement('%s:Interface'%self.UML_PREFIX)
        
    def export(self, filename):
        out = open(filename, 'w')

        xmi = XMLWriter(out)
        
        attributes = dict()
        attributes['xmi.version'] = self.XMI_VERSION
        attributes['xmlns:xmi'] = self.XMI_NAMESPACE
        attributes['xmlns:UML'] = self.UML_NAMESPACE
        
        xmi.startElement('XMI', attrs=attributes)
        
        for package in self.element_factory.select(self.select_package):
            self.handle(xmi, package)
            
        for generalization in self.element_factory.select(self.select_generalization):
            self.handle(xmi, generalization)
            
        for realization in self.element_factory.select(self.select_realization):
            self.handle(xmi, realization)
        
        xmi.endElement('XMI')
        
        log.debug(self.handled_ids)
        
    def select_package(self, element):
        return element.__class__.__name__ == 'Package'
        
    def select_generalization(self, element):
        return element.__class__.__name__ == 'Generalization'
        
    def select_realization(self, element):
        return element.__class__.name__ == 'Implementation'



#!/usr/bin/python

"""This script list classes and optionally attributes from UML model 
created with Gaphor."""

import sys
import optparse

from gaphor import Application
import gaphor.UML as UML

#Setup command line options.
usage = 'usage: %prog [options] file.gaphor'

parser = optparse.OptionParser(usage=usage)

parser.add_option('-a',\
                  '--attributes',\
                  dest='attrs',\
                  action='store_true',\
                  help='Print class attributes')

(options, args) = parser.parse_args()

if len(args) != 1:
    parser.print_help()
    sys.exit(1)

#The model file to load.
model = args[0]

#Create the Gaphor application object.
Application.init()

#Get services we need.
element_factory = Application.get_service('element_factory')
file_manager = Application.get_service('file_manager')

#Load model from file.
file_manager.load(model)

#Find all classes using factory select.
for cls in element_factory.select(lambda e: e.isKindOf(UML.Class)):
    
    print 'Found class %s' % cls.name
    
    if options.attrs:
        
        for attr in cls.ownedAttribute:
        
            print ' Attribute: %s' % attr.name



"""This module defines two visualization items - OperationItem and ClassItem."""

from gaphas.state import observed, reversible_property

from gaphor import UML
from gaphor.i18n import _

from gaphor.diagram.classifier import ClassifierItem
from gaphor.diagram.compartment import FeatureItem

class OperationItem(FeatureItem):
    """This is visualization of a class operation and is a type of 
    FeatureItem."""
    
    def render(self):
        """Render the OperationItem."""
        
        return UML.format(self.subject,\
                          visibility=True,\
                          type=True,\
                          multiplicity=True,\
                          default=True) or ''
        
class ClassItem(ClassifierItem):
    """This item visualizes a Class instance.

    A ClassItem contains two compartments (Compartment): one for
    attributes and one for operations. To add and remove such features
    the ClassItem implements the CanvasGroupable interface.
    Items can be added by callling class.add() and class.remove().
    This is used to handle CanvasItems, not UML objects!"""

    __uml__ = UML.Class, UML.Stereotype
    
    __stereotype__ = {
        'stereotype': UML.Stereotype,
        'metaclass': lambda self: (not isinstance(self.subject, UML.Stereotype)) and hasattr(self.subject, 'extension') and self.subject.extension,
    }
    
    __style__ = {
        'extra-space': 'compartment',
        'abstract-feature-font': 'sans italic 10',
    }

    def __init__(self, id=None):
        """Constructor.  Initialize the ClassItem.  This will also call the
        ClassifierItem constructor.
        
        The drawing style is set here as well.  The class item will create
        two compartments - one for attributes and another for operations."""
        
        ClassifierItem.__init__(self, id)
        self.drawing_style = self.DRAW_COMPARTMENT
        self._attributes = self.create_compartment('attributes')
        self._operations = self.create_compartment('operations')
        self._operations.use_extra_space = True

        self.watch('subject<Class>.ownedOperation', self.on_class_owned_operation)\
            .watch('subject<Class>.ownedAttribute.association', self.on_class_owned_attribute) \
            .watch('subject<Class>.ownedAttribute.name') \
            .watch('subject<Class>.ownedAttribute.isStatic') \
            .watch('subject<Class>.ownedAttribute.isDerived') \
            .watch('subject<Class>.ownedAttribute.visibility') \
            .watch('subject<Class>.ownedAttribute.lowerValue') \
            .watch('subject<Class>.ownedAttribute.upperValue') \
            .watch('subject<Class>.ownedAttribute.defaultValue') \
            .watch('subject<Class>.ownedAttribute.typeValue') \
            .watch('subject<Class>.ownedOperation.name') \
            .watch('subject<Class>.ownedOperation.isAbstract', self.on_operation_is_abstract) \
            .watch('subject<Class>.ownedOperation.isStatic') \
            .watch('subject<Class>.ownedOperation.visibility') \
            .watch('subject<Class>.ownedOperation.returnResult.lowerValue') \
            .watch('subject<Class>.ownedOperation.returnResult.upperValue') \
            .watch('subject<Class>.ownedOperation.returnResult.typeValue') \
            .watch('subject<Class>.ownedOperation.formalParameter.lowerValue') \
            .watch('subject<Class>.ownedOperation.formalParameter.upperValue') \
            .watch('subject<Class>.ownedOperation.formalParameter.typeValue') \
            .watch('subject<Class>.ownedOperation.formalParameter.defaultValue')

    def save(self, save_func):
        """Store the show- properties *before* the width/height properties,
        otherwise the classes will unintentionally grow due to "visible"
        attributes or operations."""
        
        self.save_property(save_func, 'show-attributes')
        self.save_property(save_func, 'show-operations')
        ClassifierItem.save(self, save_func)

    def postload(self):
        """Called once the ClassItem has been loaded.  First the ClassifierItem
        is "post-loaded", then the attributes and operations are
        synchronized."""
        super(ClassItem, self).postload()
        self.sync_attributes()
        self.sync_operations()

    @observed
    def _set_show_operations(self, value):
        """Sets the show operations property.  This will either show or hide
        the operations compartment of the ClassItem.  This is part of the
        show_operations property."""
        
        self._operations.visible = value
        self._operations.use_extra_space = value
        self._attributes.use_extra_space = not self._operations.visible

    show_operations = reversible_property(fget=lambda s: s._operations.visible,
            fset=_set_show_operations)
    
    @observed
    def _set_show_attributes(self, value):
        """Sets the show attributes property.  This will either show or hide
        the attributes compartment of the ClassItem.  This is part of the
        show_attributes property."""
        
        self._attributes.visible = value

    show_attributes = reversible_property(fget=lambda s: s._attributes.visible,
                               fset=_set_show_attributes)

    def _create_attribute(self, attribute):
        """Create a new attribute item.  This will create a new FeatureItem
        and assigns the specified attribute as the subject."""
        
        new = FeatureItem()
        new.subject = attribute
        new.font = self.style.feature_font
        
        self._attributes.append(new)


    def _create_operation(self, operation):
        """Create a new operation item.  This will create a new OperationItem
        and assigns the specified operation as the subject."""
        
        new = OperationItem()
        new.subject = operation
        new.font = self.style.feature_font
        
        self._operations.append(new)

    def sync_attributes(self):
        """Sync the contents of the attributes compartment with the data
        in self.subject."""
        
        owned_attributes = [a for a in self.subject.ownedAttribute if not a.association]
        self.sync_uml_elements(owned_attributes,\
                               self._attributes,\
                               self._create_attribute)

    def sync_operations(self):
        """Sync the contents of the operations compartment with the data
        in self.subject."""
        
        self.sync_uml_elements(self.subject.ownedOperation,\
                               self._operations,\
                               self._create_operation)

    def on_class_owned_attribute(self, event):
        """Event handler for owned attributes.  This will synchronize the
        attributes of this ClassItem."""
        
        if self.subject:
            self.sync_attributes()

    def on_class_owned_operation(self, event):
        """Event handler for owned operations.  This will synchronize the
        operations of this ClassItem."""
        
        if self.subject:
            self.sync_operations()

    def on_operation_is_abstract(self, event):
        """Event handler for abstract operations.  This will change the font
        of the operation."""
        
        o = [o for o in self._operations if o.subject is event.element]
        if o:
            o = o[0]
            o.font = (o.subject and o.subject.isAbstract) \
                    and self.style.abstract_feature_font or self.style.feature_font
            self.request_update()



"""
The main application window.
"""

import os.path
import gobject, gtk
from logging import getLogger

import pkg_resources
from zope import interface, component
from gaphor.interfaces import IService, IActionProvider
from interfaces import IUIComponent

from etk.docking import DockLayout, DockGroup, DockItem
from etk.docking.docklayout import add_new_group_floating

from gaphor import UML
from gaphor.core import _, inject, action, toggle_action, open_action, build_action_group, transactional
from namespace import NamespaceModel, NamespaceView
from diagramtab import DiagramTab
from toolbox import Toolbox as _Toolbox
from diagramtoolbox import TOOLBOX_ACTIONS
from etk.docking import DockItem, DockGroup, add_new_group_left, add_new_group_right, \
        add_new_group_above, add_new_group_below, add_new_group_floating, settings
from layout import deserialize

from interfaces import IDiagramTabChange
from gaphor.interfaces import IServiceEvent, IActionExecutedEvent
from gaphor.UML.event import ModelFactoryEvent
from event import DiagramTabChange, DiagramSelectionChange
from gaphor.services.filemanager import FileManagerStateChanged
from gaphor.services.undomanager import UndoManagerStateChanged
from gaphor.ui.accelmap import load_accel_map, save_accel_map

logger = getLogger(name='MainWindow')

ICONS = (
    'gaphor-24x24.png',
    'gaphor-48x48.png',
    'gaphor-96x96.png',
    'gaphor-256x256.png',
)

settings['diagrams'].expand = True
settings['diagrams'].auto_remove = False
settings['diagrams'].inherit_settings = False
settings['EtkDockGroup'].expand = False
settings['EtkDockPaned'].expand = False

STATIC_MENU_XML = """
  <ui>
    <menubar name="mainwindow">
      <menu action="%s">
        <menuitem action="%s" />
      </menu>
    </menubar>
  </ui>
"""


class MainWindow(object):
    """
    The main window for the application.
    It contains a Namespace-based tree view and a menu and a statusbar.
    """
    interface.implements(IService, IActionProvider)


    component_registry = inject('component_registry')
    properties = inject('properties')
    element_factory = inject('element_factory')
    action_manager = inject('action_manager')
    file_manager = inject('file_manager')
    ui_manager = inject('ui_manager')

    title = 'Gaphor'
    size = property(lambda s: s.properties.get('ui.window-size', (760, 580)))
    menubar_path = '/mainwindow'
    toolbar_path = '/mainwindow-toolbar'
    resizable = True

    menu_xml = """
      <ui>
        <menubar name="mainwindow">
          <menu action="file">
            <placeholder name="primary" />
            <separator />
            <menu action="file-export" />
            <menu action="file-import" />
            <separator />
            <placeholder name="secondary" />
            <placeholder name="ternary" />
            <separator />
            <menuitem action="file-quit" />
          </menu>
          <menu action="edit">
            <placeholder name="primary" />
            <placeholder name="secondary" />
            <placeholder name="ternary" />
          </menu>
          <menu action="diagram">
            <menuitem action="diagram-drawing-style" />
            <separator />
            <placeholder name="primary" />
            <placeholder name="secondary" />
            <placeholder name="ternary" />
          </menu>
          <menu action="tools">
            <placeholder name="primary" />
            <placeholder name="secondary" />
            <placeholder name="ternary" />
          </menu>
          <menu action="window">
            <placeholder name="primary" />
            <placeholder name="secondary" />
            <placeholder name="ternary" />
          </menu>
          <menu action="help">
            <placeholder name="primary" />
            <placeholder name="secondary" />
            <placeholder name="ternary" />
          </menu>
        </menubar>
        <toolbar name='mainwindow-toolbar'>
            <placeholder name="left" />
            <separator expand="true" />
            <placeholder name="right" />
        </toolbar>
      </ui>
    """

    def __init__(self):
        self.window = None
        self.model_changed = False

        # Map tab contents to DiagramTab
        #self.notebook_map = {}
        self._current_diagram_tab = None
        self.layout = None


    def init(self, app=None):
        #self.init_pygtk()
        self.init_stock_icons()
        self.init_action_group()
        self.init_ui_components()


    def init_pygtk(self):
        """
        Make sure we have GTK+ >= 2.0
        """
        import pygtk
        pygtk.require('2.0')
        del pygtk


    def init_stock_icons(self):
        # Load stock items
        import gaphor.ui.stock
        gaphor.ui.stock.load_stock_icons()


    def init_ui_components(self):
        component_registry = self.component_registry
        for ep in pkg_resources.iter_entry_points('gaphor.uicomponents'):
            log.debug('found entry point uicomponent.%s' % ep.name)
            cls = ep.load()
            if not IUIComponent.implementedBy(cls):
                raise NameError, 'Entry point %s doesn''t provide IUIComponent' % ep.name
            uicomp = cls()
            uicomp.ui_name = ep.name
            component_registry.register_utility(uicomp, IUIComponent, ep.name)
            if IActionProvider.providedBy(uicomp):
                self.action_manager.register_action_provider(uicomp)
                

    def shutdown(self):
        if self.window:
            self.window.destroy()
            self.window = None
        save_accel_map()

        cr = self.component_registry
        cr.unregister_handler(self._on_file_manager_state_changed)
        cr.unregister_handler(self._on_undo_manager_state_changed)
        cr.unregister_handler(self._new_model_content)
        #self.ui_manager.remove_action_group(self.action_group)


    def init_action_group(self):
        self.action_group = build_action_group(self)
        for name, label in (('file', '_File'),
                            ('file-export', '_Export'),
                            ('file-import', '_Import'),
                            ('edit', '_Edit'),
                            ('diagram', '_Diagram'),
                            ('tools', '_Tools'),
                            ('window', '_Window'),
                            ('help', '_Help')):
            a = gtk.Action(name, label, None, None)
            a.set_property('hide-if-empty', False)
            self.action_group.add_action(a)
        self._tab_ui_settings = None
        self.action_group.get_action('diagram-drawing-style').set_active(self.properties('diagram.sloppiness', 0) != 0)

        self.action_manager.register_action_provider(self)


    def get_filename(self):
        """
        Return the file name of the currently opened model.
        """
        return self.file_manager.filename


    def get_current_diagram_tab(self):
        """
        Get the currently opened and viewed DiagramTab, shown on the right
        side of the main window.
        See also: get_current_diagram(), get_current_diagram_view().
        """
        return self._current_diagram_tab


    def get_current_diagram(self):
        """
        Return the Diagram associated with the viewed DiagramTab.
        See also: get_current_diagram_tab(), get_current_diagram_view().
        """
        tab = self._current_diagram_tab
        return tab and tab.get_diagram()


    def get_current_diagram_view(self):
        """
        Return the DiagramView associated with the viewed DiagramTab.
        See also: get_current_diagram_tab(), get_current_diagram().
        """
        tab = self._current_diagram_tab
        return tab and tab.get_view()


    def ask_to_close(self):
        """
        Ask user to close window if the model has changed.
        The user is asked to either discard the changes, keep the
        application running or save the model and quit afterwards.
        """
        if self.model_changed:
            dialog = gtk.MessageDialog(self.window,
                    gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                    gtk.MESSAGE_WARNING,
                    gtk.BUTTONS_NONE,
                    _('Save changed to your model before closing?'))
            dialog.format_secondary_text(
                    _('If you close without saving, your changes will be discarded.'))
            dialog.add_buttons('Close _without saving', gtk.RESPONSE_REJECT,
                    gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                    gtk.STOCK_SAVE, gtk.RESPONSE_YES)
            dialog.set_default_response(gtk.RESPONSE_YES)
            response = dialog.run()
            dialog.destroy()

            if response == gtk.RESPONSE_YES:
                # On filedialog.cancel, the application should not close.
                return self.file_manager.action_save()
            return response == gtk.RESPONSE_REJECT
        return True


    def show_diagram(self, diagram):
        """
        Show a Diagram element in a new tab.
        If a tab is already open, show that one instead.
        """
        # Try to find an existing window/tab and let it get focus:
        for tab in self.get_tabs():
            if tab.get_diagram() is diagram:
                self.set_current_page(tab)
                return tab

        tab = DiagramTab(diagram)
        dock_item = tab.construct()
        dock_item.set_name('diagram-tab')
        dock_item.diagram_tab = tab
        assert dock_item.get_name() == 'diagram-tab'
        tab.set_drawing_style(self.properties('diagram.sloppiness', 0))

        self.add_tab(dock_item)

        return tab


    def open(self):

        load_accel_map()

        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.set_title(self.title)
        self.window.set_size_request(*self.size)
        self.window.set_resizable(self.resizable)

        # set default icons of gaphor windows
        icon_dir = os.path.abspath(pkg_resources.resource_filename('gaphor.ui', 'pixmaps'))
        icons = (gtk.gdk.pixbuf_new_from_file(os.path.join(icon_dir, f)) for f in ICONS)
        self.window.set_icon_list(*icons)

        self.window.add_accel_group(self.ui_manager.get_accel_group())

        
        # Create a full featured window.
        vbox = gtk.VBox()
        self.window.add(vbox)
        vbox.show()

        menubar = self.ui_manager.get_widget(self.menubar_path)
        if menubar:
            vbox.pack_start(menubar, expand=False)
        
        toolbar = self.ui_manager.get_widget(self.toolbar_path)
        if toolbar:
            vbox.pack_start(toolbar, expand=False)

        def _factory(name):
            comp = self.component_registry.get_utility(IUIComponent, name)
            logger.debug('open component %s' % str(comp))
            return comp.open()

        filename = pkg_resources.resource_filename('gaphor.ui', 'layout.xml')
        self.layout = DockLayout()

        with open(filename) as f:
            deserialize(self.layout, vbox, f.read(), _factory)
        
        self.layout.connect('item-closed', self._on_item_closed)
        self.layout.connect('item-selected', self._on_item_selected)

        vbox.show()
        # TODO: add statusbar

        self.window.show()

        self.window.connect('delete-event', self._on_window_delete)

        # We want to store the window size, so it can be reloaded on startup
        self.window.set_property('allow-shrink', True)
        self.window.connect('size-allocate', self._on_window_size_allocate)
        self.window.connect('destroy', self._on_window_destroy)
        #self.window.connect_after('key-press-event', self._on_key_press_event)

        cr = self.component_registry
        cr.register_handler(self._on_file_manager_state_changed)
        cr.register_handler(self._on_undo_manager_state_changed)
        cr.register_handler(self._new_model_content)
        # TODO: register on ElementCreate/Delete event


    def open_welcome_page(self):
        """
        Create a new tab with a textual welcome page, a sort of 101 for
        Gaphor.
        """
        pass

    def set_title(self):
        """
        Sets the window title.
        """
        filename = self.file_manager.filename
        if self.window:
            if filename:
                title = '%s - %s' % (self.title, filename)
            else:
                title = self.title
            if self.model_changed:
                title += ' *'
            self.window.set_title(title)


    # Notebook methods:

    def add_tab(self, item):
        """
        Create a new tab on the notebook with window as its contents.
        Returns: The page number of the tab.
        """
        group = list(self.layout.get_widgets('diagrams'))[0]
        group.insert_item(item)
        item.show_all()
        #item.diagram_tab = tab
        #self.notebook_map[item] = tab

    def set_current_page(self, tab):
        """
        Force a specific tab (DiagramTab) to the foreground.
        """
        for i in self.layout.get_widgets('diagram-tab'):
            if i.diagram_tab is tab:
                g = i.get_parent()
                g.set_current_item(g.item_num(i))
                return
        #for p, t in self.notebook_map.iteritems():
        #    if tab is t:
        #        num = self.notebook.page_num(p)
        #        self.notebook.set_current_page(num)
        #        return
        pass


    def get_tabs(self):
        tabs = map(lambda i: i.diagram_tab, self.layout.get_widgets('diagram-tab'))
        return tabs

    # Signal callbacks:

    @component.adapter(ModelFactoryEvent)
    def _new_model_content(self, event):
        """
        Open the toplevel element and load toplevel diagrams.
        """
        # TODO: Make handlers for ModelFactoryEvent from within the GUI obj
        for diagram in self.element_factory.select(lambda e: e.isKindOf(UML.Diagram) and not (e.namespace and e.namespace.namespace)):
            self.show_diagram(diagram)
    

    @component.adapter(FileManagerStateChanged)
    def _on_file_manager_state_changed(self, event):
        # We're only interested in file operations
        if event.service is self.file_manager:
            self.model_changed = False
            self.set_title()


    @component.adapter(UndoManagerStateChanged)
    def _on_undo_manager_state_changed(self, event):
        """
        """
        undo_manager = event.service
        if not self.model_changed and undo_manager.can_undo():
            self.model_changed = True
            self.set_title()


    def _on_window_destroy(self, window):
        """
        Window is destroyed... Quit the application.
        """
        self.window = None
        if gobject.main_depth() > 0:
            gtk.main_quit()
        cr = self.component_registry
        cr.unregister_handler(self._on_undo_manager_state_changed)
        cr.unregister_handler(self._on_file_manager_state_changed)
        cr.unregister_handler(self._new_model_content)

    def _on_window_delete(self, window = None, event = None):
        return not self.ask_to_close()

    def _clear_ui_settings(self):
        try:
            ui_manager = self.ui_manager
        except component.ComponentLookupError, e:
            log.warning('No UI manager service found')
        else:
            if self._tab_ui_settings:
                action_group, ui_id = self._tab_ui_settings
                self.ui_manager.remove_action_group(action_group)
                self.ui_manager.remove_ui(ui_id)
                self._tab_ui_settings = None

    def _on_item_closed(self, layout, group, item):
        self._clear_ui_settings()
        try:
            ui_component = item.ui_component
        except AttributeError:
            log.warning('No ui component defined on item')
        else:
            ui_component.close()
        item.destroy()

    def _on_item_selected(self, layout, group, item):
        """
        Another page (tab) is put on the front of the diagram notebook.
        A dummy action is executed.
        """
        # TODO: Here the magic happens!
        # TODO: Need to see what the active view is, or make toolbox actions global
        self._clear_ui_settings()

        # Is it a diagram view?
        try:
            tab = item.diagram_tab
        except AttributeError:
            # Not a diagram tab
            return

        self._current_diagram_tab = tab

        #content = self.notebook.get_nth_page(page_num)
        #tab = self.notebook_map.get(content)
        #assert isinstance(tab, DiagramTab), str(tab)
        
        self.ui_manager.insert_action_group(tab.action_group, -1)
        ui_id = self.ui_manager.add_ui_from_string(tab.menu_xml)
        self._tab_ui_settings = tab.action_group, ui_id
        log.debug('Menus updated with %s, %d' % self._tab_ui_settings)

        # Make sure everyone knows the selection has changed.
        self.component_registry.handle(DiagramTabChange(item), DiagramSelectionChange(tab.view, tab.view.focused_item, tab.view.selected_items))


    def _on_window_size_allocate(self, window, allocation):
        """
        Store the window size in a property.
        """
        self.properties.set('ui.window-size', (allocation.width, allocation.height))


    # Actions:

    @action(name='file-quit', stock_id='gtk-quit')
    def quit(self):
        # TODO: check for changes (e.g. undo manager), fault-save
        self.ask_to_close() and gtk.main_quit()
        self.shutdown()


    @toggle_action(name='diagram-drawing-style', label='Hand drawn style', active=False)
    def hand_drawn_style(self, active):
        """
        Toggle between straight diagrams and "hand drawn" diagram style.
        """
        if active:
            sloppiness = 0.5
        else:
            sloppiness = 0.0
        for tab in self.get_tabs():
            tab.set_drawing_style(sloppiness)
        self.properties.set('diagram.sloppiness', sloppiness)


    def create_item(self, ui_component): #, widget, title, placement=None):
        """
        Create an item for a ui component. This method can be called from UIComponents.
        """
        item = DockItem(ui_component.title)
        item.add(ui_component.open())
        group = DockGroup()
        group.insert_item(item)
        placement = ui_component.placement
        if placement:
            if placement == 'floating':
                add_new_group_floating(group, self.layout, ui_component.size)
            else:
                location = self.layout.get_widgets(placement[1])[0]
                { 'left': add_new_group_left,
                  'right': add_new_group_right,
                  'above': add_new_group_above,
                  'below': add_new_group_below }[placement[0]](location, group)
        else:
            add_new_group_floating(group)
        item.show()
        item.ui_component = ui_component
        group.show()


gtk.accel_map_add_filter('gaphor')



class Namespace(object):

    interface.implements(IUIComponent, IActionProvider)

    title = _('Namespace')
    placement = ('left', 'diagrams')

    component_registry = inject('component_registry')
    element_factory = inject('element_factory')
    ui_manager = inject('ui_manager')
    action_manager = inject('action_manager')
    main_window = inject('main_window')

    menu_xml = STATIC_MENU_XML % ('window', 'open-namespace')

    _menu_xml = """
      <ui>
        <menubar name="mainwindow">
          <menu action="diagram">
            <separator />
            <menuitem action="tree-view-create-diagram" />
            <menuitem action="tree-view-create-package" />
            <separator />
            <menuitem action="tree-view-delete-diagram" />
            <menuitem action="tree-view-delete-package" />
            <separator />
          </menu>
        </menubar>
        <popup action="namespace-popup">
          <menuitem action="tree-view-open" />
          <menuitem action="tree-view-rename" />
          <separator />
          <menuitem action="tree-view-create-diagram" />
          <menuitem action="tree-view-create-package" />
          <separator />
          <menuitem action="tree-view-delete-diagram" />
          <menuitem action="tree-view-delete-package" />
          <separator />
          <menuitem action="tree-view-refresh" />
        </popup>
      </ui>
    """
    def __init__(self):
        self._namespace = None
        self._ui_id = None
        self.action_group = build_action_group(self)


    @open_action(name='open-namespace', label=_('_Namespace'))
    def open_namespace(self):
        if not self._namespace:
            #self.main_window.create_item(self) #self.open(), self.title, self.placement)
            return self
        else:
            self._namespace.set_property('has-focus', True)

    def open(self):
        widget = self.construct()
        self.component_registry.register_handler(self.expand_root_nodes)
        return widget


    def close(self):
        if self._namespace:
            self._namespace.destroy()
            self._namespace = None

            # TODO: How to ensure stuff is removed properly from services?
            #self.ui_manager.remove_ui(self._ui_id)
        self.component_registry.unregister_handler(self.expand_root_nodes)


    def construct(self):
        self._ui_id = self.ui_manager.add_ui_from_string(self._menu_xml)

        model = NamespaceModel(self.element_factory)
        view = NamespaceView(model, self.element_factory)
        scrolled_window = gtk.ScrolledWindow()
        scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolled_window.set_shadow_type(gtk.SHADOW_IN)
        scrolled_window.add(view)
        scrolled_window.show()
        view.show()
        
        view.connect_after('event-after', self._on_view_event)
        view.connect('row-activated', self._on_view_row_activated)
        view.connect_after('cursor-changed', self._on_view_cursor_changed)
        view.connect('destroy', self._on_view_destroyed)
        self._namespace = view
        self.expand_root_nodes()

        return scrolled_window
      

    @component.adapter(ModelFactoryEvent)
    def expand_root_nodes(self, event=None):
        """
        """
        # Expand all root elements:
        self._namespace.expand_root_nodes()
        self._on_view_cursor_changed(self._namespace)


    def _on_view_event(self, view, event):
        """
        Show a popup menu if button3 was pressed on the TreeView.
        """
        # handle mouse button 3:
        if event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
            menu = self.ui_manager.get_widget('/namespace-popup')
            menu.popup(None, None, None, event.button, event.time)


    def _on_view_row_activated(self, view, path, column):
        """
        Double click on an element in the tree view.
        """
        self.action_manager.execute('tree-view-open')


    def _on_view_cursor_changed(self, view):
        """
        Another row is selected, execute a dummy action.
        """
        element = view.get_selected_element()
        self.action_group.get_action('tree-view-create-diagram').props.sensitive = isinstance(element, UML.Package)
        self.action_group.get_action('tree-view-create-package').props.sensitive = isinstance(element, UML.Package)

        self.action_group.get_action('tree-view-delete-diagram').props.visible = isinstance(element, UML.Diagram)
        self.action_group.get_action('tree-view-delete-package').props.visible = isinstance(element, UML.Package) and not element.presentation

        self.action_group.get_action('tree-view-open').props.sensitive = isinstance(element, UML.Diagram)


    def _on_view_destroyed(self, widget):
        self.close()


    def select_element(self, element):
        """
        Select an element from the Namespace view.
        The element is selected. After this an action may be executed,
        such as OpenModelElement, which will try to open the element (if it's
        a Diagram).
        """
        path = self._namespace.get_model().path_from_element(element)
        # Expand the first row:
        if len(path) > 1:
            self._namespace.expand_row(path[:-1], False)
        selection = self._namespace.get_selection()
        selection.select_path(path)
        self._on_view_cursor_changed(self._namespace)


    @action(name='tree-view-open', label='_Open')
    def tree_view_open_selected(self):
        element = self._namespace.get_selected_element()
        # TODO: Candidate for adapter?
        if isinstance(element, UML.Diagram):
            self.main_window.show_diagram(element)
        else:
            log.debug('No action defined for element %s' % type(element).__name__)


    @action(name='tree-view-rename', label=_('Rename'), accel='F2')
    def tree_view_rename_selected(self):
        view = self._namespace
        element = view.get_selected_element()
        path = view.get_model().path_from_element(element)
        column = view.get_column(0)
        cell = column.get_cell_renderers()[1]
        cell.set_property('editable', 1)
        cell.set_property('text', element.name)
        view.set_cursor(path, column, True)
        cell.set_property('editable', 0)


    @action(name='tree-view-create-diagram', label=_('_New diagram'), stock_id='gaphor-diagram')
    @transactional
    def tree_view_create_diagram(self):
        element = self._namespace.get_selected_element()
        diagram = self.element_factory.create(UML.Diagram)
        diagram.package = element

        if element:
            diagram.name = '%s diagram' % element.name
        else:
            diagram.name = 'New diagram'

        self.select_element(diagram)
        self.main_window.show_diagram(diagram)
        self.tree_view_rename_selected()


    @action(name='tree-view-delete-diagram', label=_('_Delete diagram'), stock_id='gtk-delete')
    @transactional
    def tree_view_delete_diagram(self):
        diagram = self._namespace.get_selected_element()
        m = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_QUESTION,
                              gtk.BUTTONS_YES_NO,
                              'Do you really want to delete diagram %s?\n\n'
                              'This will possibly delete diagram items\n'
                              'that are not shown in other diagrams.'
                              % (diagram.name or '<None>'))
        if m.run() == gtk.RESPONSE_YES:
            for i in reversed(diagram.canvas.get_all_items()):
                s = i.subject
                if s and len(s.presentation) == 1:
                    s.unlink()
                i.unlink
            diagram.unlink()
        m.destroy()


    @action(name='tree-view-create-package', label=_('New _package'), stock_id='gaphor-package')
    @transactional
    def tree_view_create_package(self):
        element = self._namespace.get_selected_element()
        package = self.element_factory.create(UML.Package)
        package.package = element

        if element:
            package.name = '%s package' % element.name
        else:
            package.name = 'New model'

        self.select_element(package)
        self.tree_view_rename_selected()


    @action(name='tree-view-delete-package', label=_('Delete pac_kage'), stock_id='gtk-delete')
    @transactional
    def tree_view_delete_package(self):
        package = self._namespace.get_selected_element()
        assert isinstance(package, UML.Package)
        package.unlink()


    @action(name='tree-view-refresh', label=_('_Refresh'))
    def tree_view_refresh(self):
        self._namespace.get_model().refresh()



class Toolbox(object):

    interface.implements(IUIComponent, IActionProvider)

    title = _('Toolbox')
    placement = ('left', 'diagrams')

    component_registry = inject('component_registry')
    main_window = inject('main_window')
    properties = inject('properties')

    menu_xml = """
      <ui>
        <menubar name="mainwindow">
          <menu action="diagram">
            <separator/>
            <menuitem action="reset-tool-after-create" />
            <separator/>
          </menu>
          <menu action="window">
            <menuitem action="open-toolbox" />
          </menu>
        </menubar>
      </ui>
    """
    
    def __init__(self):
        self._toolbox = None
        self.action_group = build_action_group(self)
        self.action_group.get_action('reset-tool-after-create').set_active(self.properties.get('reset-tool-after-create', True))


    @open_action(name='open-toolbox', label=_('T_oolbox'))
    def open_toolbox(self):
        if not self._toolbox:
            #self.main_window.create_item(self) #.open(), self.title, self.placement)
            return self
        else:
            self._toolbox.set_property('has-focus', True)

    def open(self):
        widget = self.construct()
        self.main_window.window.connect_after('key-press-event', self._on_key_press_event)

        self.component_registry.register_handler(self._on_diagram_tab_change)
        if self.main_window.get_current_diagram_tab():
            self.update_toolbox(self.main_window.get_current_diagram_tab().toolbox.action_group)
        return widget

    def close(self):
        if self._toolbox:
            self.component_registry.unregister_handler(self._on_diagram_tab_change)
            self._toolbox.destroy()
            self._toolbox = None


    def construct(self):
        toolbox = _Toolbox(TOOLBOX_ACTIONS)
        toolbox.show()

        toolbox.connect('destroy', self._on_toolbox_destroyed)
        self._toolbox = toolbox
        return toolbox


    def _on_key_press_event(self, view, event):
        """
        Grab top level window events and select the appropriate tool based on the event.
        """
        if event.state & gtk.gdk.SHIFT_MASK or \
	        (event.state == 0 or event.state & gtk.gdk.MOD2_MASK):
            keyval = gtk.gdk.keyval_name(event.keyval)
            self.set_active_tool(shortcut=keyval)


    def _on_toolbox_destroyed(self, widget):
        self._toolbox = None


    @toggle_action(name='reset-tool-after-create', label=_('_Reset tool'), active=False)
    def reset_tool_after_create(self, active):
        self.properties.set('reset-tool-after-create', active)


    #def _insensivate_toolbox(self):
    #    for button in self._toolbox.buttons:
    #        button.set_property('sensitive', False)

    @component.adapter(IDiagramTabChange)
    def _on_diagram_tab_change(self, event):
        self.update_toolbox(event.diagram_tab.toolbox.action_group)

    def update_toolbox(self, action_group):
        """
        Update the buttons in the toolbox. Each button should be connected
        by an action. Each button is assigned a special _action_name_
        attribute that can be used to fetch the action from the ui manager.
        """
        if not self._toolbox:
            return

        for button in self._toolbox.buttons:
            
            action_name = button.action_name
            action = action_group.get_action(action_name)
            if action:
                action.connect_proxy(button)


    def set_active_tool(self, action_name=None, shortcut=None):
        """
        Set the tool based on the name of the action
        """
        # HACK:
        toolbox = self._toolbox
        if shortcut and toolbox:
            action_name = toolbox.shortcuts.get(shortcut)
            log.debug('Action for shortcut %s: %s' % (shortcut, action_name))
            if not action_name:
                return

        self.main_window.get_current_diagram_tab().toolbox.action_group.get_action(action_name).activate()
            

# vim:sw=4:et:ai



"""
The Core module provides an entry point for Gaphor's core constructs.

An average module should only need to import this module.
"""

from gaphor.application import inject, Application
from gaphor.transaction import Transaction, transactional
from gaphor.action import action, toggle_action, radio_action, open_action, build_action_group
from gaphor.i18n import _



# vim:sw=4:et:ai



"""The element editor is a utility window used for editing elements."""

import gtk
from zope import interface
from gaphor.interfaces import IService, IActionProvider
from gaphor.core import _, inject, open_action, build_action_group
from gaphor.ui.propertyeditor import PropertyEditor
from gaphor.ui.interfaces import IUIComponent

class ElementEditor(object):
    """The ElementEditor class is a utility window used to edit UML elements.
    It will display the properties of the currently selected element in the
    diagram."""

    interface.implements(IUIComponent, IActionProvider)

    element_factory = inject('element_factory')
    properties = inject('properties')

    title = _("Element Editor")
    size = (275, -1)
    resizable = True
    placement = 'floating'
    menu_xml = """
      <ui>
        <menubar name="mainwindow">
          <menu action="edit">
            <separator />
            <menuitem action="ElementEditor:open" />
          </menu>
        </menubar>
      </ui>
    """

    def __init__(self):
        """Constructor.  Build the action group for the element editor window.
        This will place a button for opening the window in the toolbar.
        The widget attribute is a PropertyEditor."""
        
        self.action_group = build_action_group(self)
        self.property_editor = PropertyEditor()
        self.widget = self.property_editor.construct()

    @open_action(name='ElementEditor:open', label=_('Editor'), stock_id='gtk-edit', accel='<Control>e')
    def open_elementeditor(self):
        """Display the element editor when the toolbar button is toggled.  If
        active, the element editor is displayed.  Otherwise, it is hidden."""
        
        if not self.widget.get_parent():
            return self

    def open(self):
        """Display and return the PropertyEditor widget."""
        
        self.widget.show()
        return self.widget

    def close(self):
        """Hide the element editor window and deactivate the toolbar button.
        Both the widget and event parameters default to None and are
        idempotent if set."""
        
        log.debug('ElementEditor.close')
        #self.action_group.get_action('ElementEditor:open').set_active(False)
        self.widget.unparent()
        #self.dock_item.destroy()
        #return True


# vim:sw=4:et:ai



"""Unit tests for transactions in Gaphor."""

from unittest import TestCase 

from zope.component.globalregistry import base

from gaphor.application import Application
from gaphor.transaction import Transaction, transactional, TransactionError
from gaphor.event import TransactionBegin, TransactionCommit, TransactionRollback

begins = []
commits = []
rollbacks = []

def handle_begins(event):
    """Store TransactionBegin events in begins."""
    begins.append(event)

def handle_commits(event):
    """Store TransactionCommit events in commits."""
    commits.append(event)

def handle_rollback(event):
    """Store TransactionRollback events in rollbacks."""
    rollbacks.append(event)

class TransactionTestCase(TestCase):
    """Test case for transactions with the component registry 
    enabled."""

    def setUp(self):
        """Initialize Gaphor services and register transaction event
        handlers."""
        
        Application.init(services=['component_registry'])
        
        component_registry = Application.get_service('component_registry')
        
        component_registry.register_handler(handle_begins, [TransactionBegin])
        component_registry.register_handler(handle_commits, [TransactionCommit])
        component_registry.register_handler(handle_rollback, [TransactionRollback])
        
        del begins[:]
        del commits[:]
        del rollbacks[:]
        
    def tearDown(self):
        """Finished with the test case.  Unregister event handlers that
        store transaction events."""
        
        component_registry = Application.get_service('component_registry')
        
        component_registry.unregister_handler(handle_begins, [TransactionBegin])
        component_registry.unregister_handler(handle_commits, [TransactionCommit])
        component_registry.unregister_handler(handle_rollback, [TransactionRollback])


    def test_transaction_commit(self):
        """Test committing a transaction."""
        
        tx = Transaction()
        
        self.assertTrue(tx._stack, 'Transaction has no stack')
        self.assertEquals(1, len(begins), 'Incorrect number of TrasactionBegin events')
        self.assertEquals(0, len(commits), 'Incorrect number of TransactionCommit events')
        self.assertEquals(0, len(rollbacks), 'Incorrect number of TransactionRollback events')

        tx.commit()
        
        self.assertEquals(1, len(begins), 'Incorrect number of TrasactionBegin events')
        self.assertEquals(1, len(commits), 'Incorrect number of TransactionCommit events')
        self.assertEquals(0, len(rollbacks), 'Incorrect number of TransactionRollback events')
        self.assertFalse(tx._stack, 'Transaction stack is not empty')

        try:
            tx.commit()
        except TransactionError:
            pass
        else:
            self.fail('Commit should not have succeeded')


    def test_transaction_rollback(self):
        """Test rolling back a transaction."""
        
        tx = Transaction()
        
        self.assertTrue(tx._stack, 'Transaction has no stack')
        self.assertEquals(1, len(begins), 'Incorrect number of TrasactionBegin events')
        self.assertEquals(0, len(commits), 'Incorrect number of TransactionCommit events')
        self.assertEquals(0, len(rollbacks), 'Incorrect number of TransactionRollback events')

        tx.rollback()
        
        self.assertEquals(1, len(begins), 'Incorrect number of TrasactionBegin events')
        self.assertEquals(0, len(commits), 'Incorrect number of TransactionCommit events')
        self.assertEquals(1, len(rollbacks), 'Incorrect number of TransactionRollback events')

        self.assertFalse(tx._stack, 'Transaction stack is not empty')


    def test_transaction_commit_after_rollback(self):
        """Test committing one transaction after rolling back another
        transaction."""
        
        tx1 = Transaction()
        tx2 = Transaction()

        tx2.rollback()
        tx1.commit()
        
        self.assertEquals(1, len(begins), 'Incorrect number of TrasactionBegin events')
        self.assertEquals(0, len(commits), 'Incorrect number of TransactionCommit events')
        self.assertEquals(1, len(rollbacks), 'Incorrect number of TransactionRollback events')


    def test_transaction_stack(self):
        """Test the transaction stack."""
        
        tx1 = Transaction()
        tx2 = Transaction()

        try:
            tx1.commit()
        except TransactionError, e:
            pass
        else:
            self.fail('Commit should not have succeeded')


    def test_transaction_context(self):
        """Test the transaction context manager."""
        
        with Transaction() as tx:
            
            self.assertTrue(isinstance(tx, Transaction), 'Context is not a Transaction instance')
            self.assertTrue(Transaction._stack, 'Transaction instance has no stack inside a context')
            
        self.assertFalse(Transaction._stack, 'Transaction stack should be empty')


    def test_transaction_context_error(self):
        """Test the transaction context manager with errors."""
        
        try:
            with Transaction():
                raise TypeError('transaction error')
        except TypeError, e:
            self.assertEquals('transaction error', str(e), 'Transaction context manager did no raise correct exception')
        else:
            self.fail('Transaction context manager did not raise exception when it should have')
            
class TransactionWithoutComponentRegistryTestCase(TestCase):
    """Test case for transactions with no component registry."""

    def test_transaction(self):
        """Test basic transaction functionality."""
        
        tx = Transaction()
        tx.rollback()

        tx = Transaction()
        tx.commit()



"""
Service dedicated to exporting diagrams to a varyity of file formats.
"""

import os
import cairo

from zope import interface, component

from logging import getLogger
from gaphor.core import _, inject, action, build_action_group
from gaphor.interfaces import IService, IActionProvider
from gaphor.ui.filedialog import FileDialog
from gaphor.ui.questiondialog import QuestionDialog

from gaphas.view import View
from gaphas.painter import ItemPainter, BoundingBoxPainter
from gaphas.freehand import FreeHandPainter
from gaphas.geometry import Rectangle

class DiagramExportManager(object):
    """
    Service for exporting diagrams as images (SVG, PNG, PDF).
    """

    interface.implements(IService, IActionProvider)

    main_window = inject('main_window')
    properties = inject('properties')
    logger = getLogger('ExportManager')

    menu_xml = """
      <ui>
        <menubar action="mainwindow">
          <menu action="file">
            <menu action="file-export">
              <menuitem action="file-export-svg" />
              <menuitem action="file-export-png" />
              <menuitem action="file-export-pdf" />
              <separator />
            </menu>
          </menu>
        </menubar>
      </ui>
    """

    def __init__(self):
        self.action_group = build_action_group(self)

    def init(self, app):
        pass

    def shutdown(self):
        pass

    def update(self):
        
        self.logger.info('Updating')
        
        tab = self.get_window().get_current_diagram_tab()
        self.sensitive = tab and True or False

    def save_dialog(self, diagram, title, ext):
        
        filename = (diagram.name or 'export') + ext
        file_dialog = FileDialog(title, action='save', filename=filename)
        
        save = False
        while True:
            filename = file_dialog.selection
            if os.path.exists(filename):
                question = _("The file %s already exists. Do you want to "\
                             "replace it with the file you are exporting "\
                             "to?") % filename
                question_dialog = QuestionDialog(question)
                answer = question_dialog.answer
                question_dialog.destroy()
                if answer:
                    save = True
                    break
            else:
                save = True
                break
                
        file_dialog.destroy()
        
        if save and filename:
            return filename                
        
    def update_painters(self, view):
        
        self.logger.info('Updating painters')
        self.logger.debug('View is %s' % view)
        
        sloppiness = self.properties('diagram.sloppiness', 0)
        
        self.logger.debug('Sloppiness is %s' % sloppiness)
        
        if sloppiness:
            view.painter = FreeHandPainter(ItemPainter(), sloppiness)
            view.bounding_box_painter = FreeHandPainter(BoundingBoxPainter(), sloppiness)
        else:
            view.painter = ItemPainter()

    def save_svg(self, filename, canvas):
        
        self.logger.info('Exporting to SVG')
        self.logger.debug('SVG path is %s' % filename)
        
        view = View(canvas)

        self.update_painters(view)

        # Update bounding boxes with a temporaly CairoContext
        # (used for stuff like calculating font metrics)
        tmpsurface = cairo.ImageSurface(cairo.FORMAT_ARGB32, 0, 0)
        tmpcr = cairo.Context(tmpsurface)
        view.update_bounding_box(tmpcr)
        tmpcr.show_page()
        tmpsurface.flush()

        w, h = view.bounding_box.width, view.bounding_box.height
        surface = cairo.SVGSurface(filename, w, h)
        cr = cairo.Context(surface)
        view.matrix.translate(-view.bounding_box.x, -view.bounding_box.y)
        view.paint(cr)
        cr.show_page()
        surface.flush()
        surface.finish()


    def save_png(self, filename, canvas):
        
        self.logger.info('Exporting to PNG')
        self.logger.debug('PNG path is %s' % filename)
        
        view = View(canvas)

        self.update_painters(view)

        # Update bounding boxes with a temporaly CairoContext
        # (used for stuff like calculating font metrics)
        tmpsurface = cairo.ImageSurface(cairo.FORMAT_ARGB32, 0, 0)
        tmpcr = cairo.Context(tmpsurface)
        view.update_bounding_box(tmpcr)
        tmpcr.show_page()
        tmpsurface.flush()

        w, h = view.bounding_box.width, view.bounding_box.height
        surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, int(w+1), int(h+1))
        cr = cairo.Context(surface)
        view.matrix.translate(-view.bounding_box.x, -view.bounding_box.y)
        view.paint(cr)
        cr.show_page()
        surface.write_to_png(filename)

    def save_pdf(self, filename, canvas):
        
        self.logger.info('Exporting to PDF')
        self.logger.debug('PDF path is %s' % filename)
        
        view = View(canvas)

        self.update_painters(view)

        # Update bounding boxes with a temporaly CairoContext
        # (used for stuff like calculating font metrics)
        tmpsurface = cairo.ImageSurface(cairo.FORMAT_ARGB32, 0, 0)
        tmpcr = cairo.Context(tmpsurface)
        view.update_bounding_box(tmpcr)
        tmpcr.show_page()
        tmpsurface.flush()

        w, h = view.bounding_box.width, view.bounding_box.height
        surface = cairo.PDFSurface(filename, w, h)
        cr = cairo.Context(surface)
        view.matrix.translate(-view.bounding_box.x, -view.bounding_box.y)
        view.paint(cr)
        cr.show_page()
        surface.flush()
        surface.finish()

    @action(name='file-export-svg', label='Export to SVG',
            tooltip='Export the diagram to SVG')
    def save_svg_action(self):
        title = 'Export diagram to SVG'
        ext = '.svg'
        diagram = self.main_window.get_current_diagram()
        filename = self.save_dialog(diagram, title, ext)
        if filename:
            self.save_svg(filename, diagram.canvas)


    @action(name='file-export-png', label='Export to PNG',
            tooltip='Export the diagram to PNG')
    def save_png_action(self):
        title = 'Export diagram to PNG'
        ext = '.png'
        diagram = self.main_window.get_current_diagram()
        filename = self.save_dialog(diagram, title, ext)
        if filename:
            self.save_png(filename, diagram.canvas)


    @action(name='file-export-pdf', label='Export to PDF',
            tooltip='Export the diagram to PDF')
    def save_pdf_action(self):
        title = 'Export diagram to PDF'
        ext = '.pdf'
        diagram = self.main_window.get_current_diagram()
        filename = self.save_dialog(diagram, title, ext)
        if filename:
            self.save_pdf(filename, diagram.canvas)


# vim:sw=4:et:




"""Internationalization (i18n) support for Gaphor.

Here the _() function is defined that is used to translate text into
your native language."""

__all__ = [ '_' ]

import os

import gettext
import pkg_resources

localedir = os.path.join(pkg_resources.get_distribution('gaphor').location, \
                         'gaphor', 'data', 'locale')

try:
    
    catalog = gettext.Catalog('gaphor', localedir=localedir)
    _ = catalog.gettext
    
except IOError, e:
    
    def _(s): return s




"""
Test implementation (interface realization) item connectors.
"""

from gaphor import UML
from gaphor.diagram import items
from gaphor.tests import TestCase

class ImplementationTestCase(TestCase):
    def test_non_interface_glue(self):
        """Test non-interface glueing with implementation
        """
        impl = self.create(items.ImplementationItem)
        clazz = self.create(items.ClassItem, UML.Class)

        glued = self.allow(impl, impl.head, clazz)
        # connecting head to non-interface item is disallowed
        self.assertFalse(glued)


    def test_interface_glue(self):
        """Test interface glueing with implementation
        """
        iface = self.create(items.InterfaceItem, UML.Interface)
        impl = self.create(items.ImplementationItem)

        glued = self.allow(impl, impl.head, iface)
        self.assertTrue(glued)


    def test_classifier_glue(self):
        """Test classifier glueing with implementation
        """
        impl = self.create(items.ImplementationItem)
        clazz = self.create(items.ClassItem, UML.Class)

        glued = self.allow(impl, impl.tail, clazz)
        self.assertTrue(glued)


    def test_connection(self):
        """Test connection of class and interface with implementation
        """
        iface = self.create(items.InterfaceItem, UML.Interface)
        impl = self.create(items.ImplementationItem)
        clazz = self.create(items.ClassItem, UML.Class)

        self.connect(impl, impl.head, iface)
        self.connect(impl, impl.tail, clazz)

        # check the datamodel
        self.assertTrue(isinstance(impl.subject, UML.Implementation))
        ct = self.get_connected(impl.head)
        self.assertTrue(ct is iface)
        self.assertTrue(impl.subject is not None)
        self.assertTrue(impl.subject.contract[0] is iface.subject)
        self.assertTrue(impl.subject.implementatingClassifier[0] is clazz.subject)


    def test_reconnection(self):
        """Test reconnection of class and interface with implementation
        """
        iface = self.create(items.InterfaceItem, UML.Interface)
        c1 = self.create(items.ClassItem, UML.Class)
        c2 = self.create(items.ClassItem, UML.Class)
        impl = self.create(items.ImplementationItem)

        # connect: iface -> c1
        self.connect(impl, impl.head, iface)
        self.connect(impl, impl.tail, c1)

        s = impl.subject

        # reconnect: iface -> c2
        self.connect(impl, impl.tail, c2)

        self.assertSame(s, impl.subject)
        self.assertEquals(1, len(impl.subject.contract))
        self.assertEquals(1, len(impl.subject.implementatingClassifier))
        self.assertTrue(iface.subject in impl.subject.contract)
        self.assertTrue(c2.subject in impl.subject.implementatingClassifier)
        self.assertTrue(c1.subject not in impl.subject.implementatingClassifier, impl.subject.implementatingClassifier)



# vim:sw=4:et:ai




import unittest

import gaphor.UML as UML

class ClassesTestCase(unittest.TestCase):
    
    def setUp(self):
        
        self.factory = UML.ElementFactory()

    def tearDown(self):
        
        del self.factory    
    
    def test_association(self):
        
        """Testing Association elements in the meta-model"""
        
        try:
            
            element = self.factory.create(UML.Association)
            
        except AttributeError:
            
            self.fail('Association elements are not part of the meta-model')
            
        self.assertFalse(element.isDerived, 'The isDerived property should default to False - %s' % element.isDerived)
        
        property1 = self.factory.create(UML.Property)
        property2 = self.factory.create(UML.Property)
        
        element.memberEnd = property1
        element.memberEnd = property2
        
        element.ownedEnd = property1
        
        element.navigableOwnedEnd = property1
        
        self.assertTrue(property1 in element.member, 'Namespace.member does not contain memberEnd - %s' % element.member)
        self.assertTrue(property2 in element.member, 'Namespace.member does not contain memberEnd - %s' % element.member)
        
        self.assertTrue(property1 in element.feature, 'Classifier.feature does not contain ownedEnd - %s' % element.feature)
        self.assertTrue(property1 in element.ownedMember, 'Namespace.ownedMember does not contain ownedEnd - %s' % element.ownedEnd)
        
        self.assertTrue(property1 in element.ownedEnd, 'Association.ownedEnd does not contain navigableOwnedEnd - %s' % element.ownedEnd)
        
#    def test_association_class(self):
#        try:
#            element = self.factory.create(UML.AssociationClass)
#        except AttributeError:
#            self.fail('AssociationClass elements are not part of the meta-model')
        
    def test_class(self):
        
        """Testing Class elements in the meta-model"""
        
        try:
            
            element = self.factory.create(UML.Class)
            
        except AttributeError:
            
            self.fail('Class elements are not part of the meta-model')
            
        property1 = self.factory.create(UML.Property)
        operation1 = self.factory.create(UML.Operation)
        
        element.ownedAttribute = property1
        element.ownedOperation = operation1
        
        self.assertTrue(property1 in element.attribute, 'Classifier.attribute does not contain ownedAttribute - %s' % element.attribute)
        self.assertTrue(property1 in element.ownedMember, 'Namespace.ownedMember does not contain ownedAttribute - %s' % element.ownedMember)
        
        self.assertTrue(operation1 in element.feature, 'Classifier.feature does not contain ownedOperation - %s' % element.feature)
        self.assertTrue(operation1 in element.ownedMember, 'Namespace.ownedMember does not contain ownedOperation' % element.ownedMember)
        
    def test_comment(self):
        
        """Testing Comment elements in the meta-model"""
        
        try:
            
            element = self.factory.create(UML.Comment)
            
        except AttributeError:
            
            self.fail('Comment elements are not part of the meta-model')
            
        element.body = 'Comment body'
        
        self.assertTrue(element.body == 'Comment body', 'Incorrect comment body - %s' % element.body)
        
        annotatedElement = self.factory.create(UML.Class)
        
        element.annotatedElement = annotatedElement
        
        self.assertTrue(annotatedElement in element.annotatedElement, 'Incorrect annotated element - %s' % element.annotatedElement)
        
    def test_constraint(self):
        
        """Testing Constraint elements in the meta-model"""
        
        try:
            
            element = self.factory.create(UML.Constraint)
            
        except AttributeError:
            
            self.fail('Constraint elements are not part of the meta-model')
            
        constrainedElement = self.factory.create(UML.Class)
        
        element.constrainedElement = constrainedElement
        element.specification = 'Constraint specification'
        
        self.assertTrue(constrainedElement in element.constrainedElement, 'Constraint.constrainedElement does not contain the correct element - %s' % element.constrainedElement)
        self.assertTrue(element.specification == 'Constraint specification', 'Constraint.specification is incorrect - %s' % element.specification)
        
    def test_dependency(self):
        
        """Testing Dependency elements in the meta-model"""
        
        try:
            
            element = self.factory.create(UML.Dependency)
            
        except AttributeError:
            
            self.fail('Dependency elements are not part of the meta-model')
            
        client = self.factory.create(UML.Package)
        supplier = self.factory.create(UML.Package)
        
        element.client = client
        element.supplier = supplier
        
        self.assertTrue(client in element.source, 'DirectedRelationship.source does not contain client - %s' % element.client)
        self.assertTrue(supplier in element.target, 'DirectedRelationship.target does not contain supplier - %s' % element.supplier)
        
    def test_element_import(self):
        
        try:
            
            element = self.factory.create(UML.ElementImport)
            
        except AttributeError:
            
            self.fail('ElementImport elements are not part of the meta-model')
        
    def test_enumeration(self):
        
        try:
            
            element = self.factory.create(UML.Enumeration)
            
        except AttributeError:
            
            self.fail('Enumeration elements are not part of the meta-model')
        
    def test_generalization(self):
        
        try:
            
            element = self.factory.create(UML.Generalization)
            
        except AttributeError:
            
            self.fail('Generalization elements are not part of the meta-model')
            
    def test_interface(self):
        
        try:
            
            element = self.factory.create(UML.Interface)
            
        except AttributeError:
            
            self.fail('Interface elements are not part of the meta-model')
            
    def test_namespace(self):
        
        try:
            
            element = self.factory.create(UML.Namespace)
            
        except AttributeError:
            
            self.fail('Namespace elements are not part of the meta-model')
            
    def test_operation(self):
        
        try:
            
            element = self.factory.create(UML.Operation)
            
        except AttributeError:
            
            self.fail('Operation elements are not part of the meta-model')
            
    def test_package(self):
        
        try:
            
            element = self.factory.create(UML.Package)
            
        except AttributeError:
            
            self.fail('Package elements are not part of the meta-model')
            
    def test_parameter(self):
        
        try:
            
            element = self.factory.create(UML.Parameter)
            
        except AttributeError:
            
            self.fail('Parameter elements are not part of the meta-model')
            
    def test_property(self):
        
        try:
            
            element = self.factory.create(UML.Property)
            
        except AttributeError:
            
            self.fail('Property elements are not part of the meta-model')
            
    def test_realization(self):
        
        try:
            
            element = self.factory.create(UML.Realization)
            
        except AttributeError:
            
            self.fail('Realization elements are not part of the meta-model')
        

class Uml2TestCase(unittest.TestCase):

    def test_ids(self):
        factory = UML.ElementFactory()
        c = factory.create(UML.Class)
        assert c.id
        p = factory.create_as(UML.Class, id=False)
        assert p.id is False, p.id


    def test1(self):
        factory = UML.ElementFactory()
        c = factory.create(UML.Class)
        p = factory.create(UML.Package)
        c.package = p
        self.assertEquals(c.package, p)
        self.assertEquals(c.namespace, p)
        self.failUnless(c in p.ownedElement)
        
    def testOwnedMember_Unlink(self):
        factory = UML.ElementFactory()
        c = factory.create(UML.Class)
        p = factory.create(UML.Package)
        c.package = p

        c.unlink()

        self.assertEquals([p], factory.lselect())


#    def test_lower_upper(self):
#        """
#        Test MultiplicityElement.{lower|upper}
#        """
#        assert UML.MultiplicityElement.lowerValue in UML.MultiplicityElement.lower.subsets
#
#        e = UML.MultiplicityElement()
#        e.lowerValue = '2'
#        assert e.lower == '2', e.lower
#
#        assert UML.MultiplicityElement.upperValue in UML.MultiplicityElement.upper.subsets
#
#        e.upperValue = 'up'
#        assert UML.MultiplicityElement.upper.version == 4, UML.MultiplicityElement.upper.version
#        assert e.upper == 'up'
#        e.upperValue = 'down'
#        assert UML.MultiplicityElement.upper.version == 5, UML.MultiplicityElement.upper.version
#        assert e.upper == 'down', e.upper
#
#        # TODO: test signal handling

    def test_property_is_composite(self):
        p = UML.Property()
        assert p.isComposite == False, p.isComposite
        p.aggregation = 'shared'
        assert p.isComposite == False, p.isComposite
        p.aggregation = 'composite'
        assert p.isComposite == True, p.isComposite


    def test_association_endType(self):
        factory = UML.ElementFactory()
        c1 = UML.Class()
        c2 = UML.Class()
        a = UML.Association()
        a.memberEnd = UML.Property()
        a.memberEnd = UML.Property()
        a.memberEnd[0].type = c1
        a.memberEnd[1].type = c2
        c1.ownedAttribute = a.memberEnd[0]
        c2.ownedAttribute = a.memberEnd[1]

        assert c1 in a.endType
        assert c2 in a.endType

        c3 = UML.Class()
        a.memberEnd[1].type = c3

        assert c1 in a.endType
        assert c3 in a.endType



    def test_property_navigability(self):
        factory = UML.ElementFactory()
        p = factory.create(UML.Property)
        assert p.navigability is None

        c1 = factory.create(UML.Class)
        c2 = factory.create(UML.Class)
        a = UML.model.create_association(factory, c1, c2)
        assert a.memberEnd[0].navigability is None
        assert a.memberEnd[1].navigability is None

        UML.model.set_navigability(a, a.memberEnd[0], True)
        assert a.memberEnd[0].navigability is True
        assert a.memberEnd[1].navigability is None

        UML.model.set_navigability(a, a.memberEnd[0], False)
        assert a.memberEnd[0].navigability is False
        assert a.memberEnd[1].navigability is None

    def test_namedelement_qualifiedname(self):
        factory = UML.ElementFactory()
        p = factory.create(UML.Package)
        p.name = 'Package'
        c = factory.create(UML.Class)
        c.name = 'Class'

        self.assertEquals(('Class',), c.qualifiedName)

        p.ownedClassifier = c

        self.assertEquals(('Package', 'Class'), c.qualifiedName)


    def test_extension_metaclass(self):
        factory = UML.ElementFactory()
        c = factory.create(UML.Class)
        c.name = 'Class'
        s = factory.create(UML.Stereotype)
        s.name = 'Stereotype'

        e = UML.model.create_extension(factory, c, s)

        self.assertEquals(c, e.metaclass)

    def test_metaclass_extension(self):
        factory = UML.ElementFactory()
        c = factory.create(UML.Class)
        c.name = 'Class'
        s = factory.create(UML.Stereotype)
        s.name = 'Stereotype'

        self.assertEquals([], c.extension)
        self.assertEquals([], s.extension)

        e = UML.model.create_extension(factory, c, s)

        print e.memberEnd
        self.assertEquals([e], c.extension)
        self.assertEquals([], s.extension)
        assert e.ownedEnd.type is s

    def test_operation_parameter_deletion(self):
        factory = UML.ElementFactory()
        self.assertEquals(0, len(factory.lselect()))

        c = factory.create(UML.Class)
        c.name = 'Class'
        o = factory.create(UML.Operation)
        c.ownedOperation = o
        UML.parse(o, 'a(x: int, y: int)')

        c.unlink()

        self.assertEquals(0, len(factory.lselect()), factory.lselect())


# vim:sw=4:et:ai
